@@ -1,5 +1,5 @@
 #
-# demo.tcl,v 1.100 2006/04/08 00:48:39 jenglish Exp
+# demo.tcl,v 1.103 2006/08/20 19:00:07 jenglish Exp
 #
 # Tile widget set -- widget demo
 #
@@ -799,14 +799,22 @@ ttk::button $cmd.help -text Help -command showHelp
 
 proc showHelp {} {
     if {![winfo exists .helpDialog]} {
-	ttk::dialog .helpDialog -type ok -message "Tile demo" \
-	    -detail "Tile version [package provide tile]"
+	lappend detail "Tile version [package provide tile]"
+	lappend detail "Tile library: $::tile::library"
+	lappend detail "[package ifneeded tile [package provide tile]]"
+	ttk::dialog .helpDialog -type ok -icon info \
+	    -message "Tile demo" -detail [join $detail \n]
     }
 }
 
 grid x $cmd.close $cmd.help -pady 6 -padx 6
 grid columnconfigure $cmd 0 -weight 1
 
+## Status bar (to demonstrate size grip)
+#
+set statusbar [ttk::frame $BASE.statusbar]
+pack [ttk::sizegrip $statusbar.grip] -side right -anchor se
+
 ## Accelerators:
 #
 bind $::ROOT <KeyPress-Escape>	[list event generate $cmd.close <<Invoke>>]
@@ -864,6 +872,7 @@ $menu add cascade -label "Theme" -underline 3 -menu [makeThemeMenu $menu.theme]
 ### Main window layout.
 #
 
+pack $BASE.statusbar -side bottom -expand false -fill x
 pack $BASE.command -side bottom -expand false -fill x
 pack $BASE.tbar_styled -side top -expand false -fill x
 pack $BASE.tbar_orig -side top -expand false -fill x
