@@ -1,4 +1,4 @@
-/* button.c,v 1.47 2006/01/23 02:38:04 jenglish Exp
+/* button.c,v 1.48 2006/08/04 23:22:50 patthoyts Exp
  * Copyright (c) 2003, Joe English
  *
  * Tile widget set: label, button, checkbutton, radiobutton, and 
@@ -348,6 +348,7 @@ static WidgetCommandSpec LabelCommands[] =
     { "cget",		WidgetCgetCommand },
     { "instate",	WidgetInstateCommand },
     { "state",  	WidgetStateCommand },
+    { "identify",	WidgetIdentifyCommand },
     { NULL, NULL }
 };
 
@@ -449,6 +450,7 @@ static WidgetCommandSpec ButtonCommands[] =
     { "invoke",		ButtonInvokeCommand },
     { "instate",	WidgetInstateCommand },
     { "state",  	WidgetStateCommand },
+    { "identify",	WidgetIdentifyCommand },
     { NULL, NULL }
 };
 
@@ -639,6 +641,7 @@ static WidgetCommandSpec CheckbuttonCommands[] =
     { "invoke",		CheckbuttonInvokeCommand },
     { "instate",	WidgetInstateCommand },
     { "state",  	WidgetStateCommand },
+    { "identify",	WidgetIdentifyCommand },
     /* MISSING: select, deselect, toggle */
     { NULL, NULL }
 };
@@ -809,6 +812,7 @@ static WidgetCommandSpec RadiobuttonCommands[] =
     { "invoke",		RadiobuttonInvokeCommand },
     { "instate",	WidgetInstateCommand },
     { "state",  	WidgetStateCommand },
+    { "identify",	WidgetIdentifyCommand },
     /* MISSING: select, deselect */
     { NULL, NULL }
 };
@@ -871,6 +875,7 @@ static WidgetCommandSpec MenubuttonCommands[] =
     { "cget",		WidgetCgetCommand },
     { "instate",	WidgetInstateCommand },
     { "state",  	WidgetStateCommand },
+    { "identify",	WidgetIdentifyCommand },
     { NULL, NULL }
 };
 
