@@ -1,5 +1,5 @@
 /*
- * clamTheme.c,v 1.24 2005/09/04 17:24:29 jenglish Exp
+ * clamTheme.c,v 1.25 2006/07/30 18:16:51 patthoyts Exp
  *
  * Copyright (C) 2004 Joe English
  *
@@ -139,7 +139,7 @@ static void BorderElementGeometry(
     int borderWidth = 2;
     Tk_GetPixelsFromObj(NULL, tkwin, border->borderWidthObj, &borderWidth);
     if (borderWidth == 1) ++borderWidth;
-    *paddingPtr = Ttk_UniformPadding(borderWidth);
+    *paddingPtr = Ttk_UniformPadding((short)borderWidth);
 }
 
 static void BorderElementDraw(
