@@ -1,5 +1,5 @@
 /*
- * entry.c,v 1.39 2006/04/17 15:56:05 jenglish Exp
+ * entry.c,v 1.41 2006/10/03 18:22:01 jenglish Exp
  *
  * DERIVED FROM: tk/generic/tkEntry.c r1.35.
  *
@@ -204,24 +204,6 @@ static Tk_OptionSpec EntryOptionSpecs[] =
 	NULL, Tk_Offset(Entry, entry.styleData.backgroundObj), -1,
 	TK_OPTION_NULL_OK,0,0},
 
-#if UNDOCUMENTED_OPTIONS
-    {TK_OPTION_COLOR, "-insertcolor", "insertColor", "TextColor",
-	NULL, Tk_Offset(Entry, entry.styleData.insertColorObj), -1,
-	TK_OPTION_NULL_OK,0,0},
-    {TK_OPTION_PIXELS, "-insertwidth", "insertWidth", "InsertWidth",
-	"1", Tk_Offset(Entry, entry.styleData.insertWidthObj), -1,
-        0, 0, 0},
-    {TK_OPTION_BORDER, "-selectbackground", "selectBackground", "AccentColor",
-        NULL, Tk_Offset(Entry, entry.styleData.selBorderObj), -1,
-        TK_OPTION_NULL_OK,0,0},
-    {TK_OPTION_COLOR, "-selectforeground", "selectForeground", "ContrastColor",
-	NULL, Tk_Offset(Entry, entry.styleData.selForegroundObj), -1,
-	TK_OPTION_NULL_OK,0,0},
-    {TK_OPTION_PIXELS, "-selectborderwidth","selectBorderWidth","BorderWidth",
-	NULL, Tk_Offset(Entry, entry.styleData.selBorderWidthObj), -1,
-        TK_OPTION_NULL_OK, 0, 0},
-#endif
-
     WIDGET_INHERIT_OPTIONS(CoreOptionSpecs)
 };
 
@@ -1391,8 +1373,8 @@ EntryBBoxCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
 {
     Entry *entryPtr = recordPtr;
-    int index, x, y, width, height;
-    char buf[TCL_INTEGER_SPACE * 4];
+    Ttk_Box b;
+    int index;
 
     if (objc != 3) {
 	Tcl_WrongNumArgs(interp, 2, objv, "index");
@@ -1404,11 +1386,11 @@ EntryBBoxCommand(
     if ((index == entryPtr->entry.numChars) && (index > 0)) {
 	index--;
     }
-    Tk_CharBbox(entryPtr->entry.textLayout, index, &x, &y, &width, &height);
-    sprintf(buf, "%d %d %d %d",
-		 x + entryPtr->entry.layoutX, y + entryPtr->entry.layoutY,
-		 width, height);
-    Tcl_SetResult(interp, buf, TCL_VOLATILE);
+    Tk_CharBbox(entryPtr->entry.textLayout, index, 
+	    &b.x, &b.y, &b.width, &b.height);
+    b.x += entryPtr->entry.layoutX;
+    b.y += entryPtr->entry.layoutY;
+    Tcl_SetObjResult(interp, Ttk_NewBoxObj(b));
     return TCL_OK;
 }
 
