@@ -1,4 +1,4 @@
-/* frame.c,v 1.48 2006/04/22 20:43:00 jenglish Exp
+/* frame.c,v 1.49 2006/08/04 23:22:50 patthoyts Exp
  * Copyright (c) 2004, Joe English
  *
  * Tile widget set: frame and labelframe widgets
@@ -56,6 +56,7 @@ static WidgetCommandSpec FrameCommands[] =
     { "cget",		WidgetCgetCommand },
     { "instate",	WidgetInstateCommand },
     { "state",  	WidgetStateCommand },
+    { "identify",   WidgetIdentifyCommand },
     { NULL, NULL }
 };
 
