@@ -1,4 +1,4 @@
-/* label.c,v 1.14 2005/10/08 15:53:42 jenglish Exp
+/* label.c,v 1.16 2006/04/17 01:53:13 jenglish Exp
  *
  * Tile widget set: text, image, and label elements.
  *
@@ -32,6 +32,7 @@ typedef struct
     Tcl_Obj	*backgroundObj;
     Tcl_Obj	*underlineObj;
     Tcl_Obj	*widthObj;
+    Tcl_Obj	*anchorObj;
     Tcl_Obj	*justifyObj;
     Tcl_Obj	*wrapLengthObj;
     Tcl_Obj     *embossedObj;
@@ -64,6 +65,8 @@ static Ttk_ElementOptionSpec TextElementOptions[] =
 	Tk_Offset(TextElement,underlineObj), "-1"},
     { "-width", TK_OPTION_INT,
 	Tk_Offset(TextElement,widthObj), "-1"},
+    { "-anchor", TK_OPTION_ANCHOR,
+	Tk_Offset(TextElement,anchorObj), "center"},
     { "-justify", TK_OPTION_JUSTIFY,
 	Tk_Offset(TextElement,justifyObj), "left" },
     { "-wraplength", TK_OPTION_PIXELS,
@@ -135,6 +138,7 @@ static void TextDraw(TextElement *text, Tk_Window tkwin, Drawable d, Ttk_Box b)
     int lastChar = -1;
     XGCValues gcValues;
     GC gc1, gc2;
+    Tk_Anchor anchor = TK_ANCHOR_CENTER;
 
     gcValues.font = Tk_FontId(text->tkfont);
     gcValues.foreground = color->pixel;
@@ -142,6 +146,12 @@ static void TextDraw(TextElement *text, Tk_Window tkwin, Drawable d, Ttk_Box b)
     gcValues.foreground = WhitePixelOfScreen(Tk_Screen(tkwin));
     gc2 = Tk_GetGC(tkwin, GCFont | GCForeground, &gcValues);
 
+    /* 
+     * Place text according to -anchor:
+     */
+    Tk_GetAnchorFromObj(NULL, text->anchorObj, &anchor);
+    b = Ttk_AnchorBox(b, text->width, text->height, anchor);
+
     /*
      * Clip text if it's too wide:
      * @@@ BUG: This will overclip multi-line text.
@@ -223,8 +233,7 @@ static void ImageTextElementDraw(
 	return;
 
     XFillRectangle(Tk_Display(tkwin), d,
-	Tk_3DBorderGC(tkwin, bd, TK_3D_FLAT_GC),
-	b.x, b.y, text->width, text->height);
+	Tk_3DBorderGC(tkwin, bd, TK_3D_FLAT_GC), b.x, b.y, b.width, b.height);
 
     TextDraw(text, tkwin, d, b);
     TextCleanup(text);
@@ -469,11 +478,10 @@ static void ImageElementDraw(
 typedef struct
 {
     /*
-     * Element resources:
+     * Element options:
      */
     Tcl_Obj		*compoundObj;
     Tcl_Obj		*spaceObj;
-    Tcl_Obj		*anchorObj;
     TextElement 	text;
     ImageElement	image;
 
@@ -489,8 +497,6 @@ static Ttk_ElementOptionSpec LabelElementOptions[] =
 {
     { "-compound", TK_OPTION_ANY,
 	Tk_Offset(LabelElement,compoundObj), "none" },
-    { "-anchor", TK_OPTION_ANCHOR,
-	Tk_Offset(LabelElement,anchorObj), "center"},
     { "-space", TK_OPTION_PIXELS,
 	Tk_Offset(LabelElement,spaceObj), "4" },
 
@@ -509,6 +515,8 @@ static Ttk_ElementOptionSpec LabelElementOptions[] =
 	Tk_Offset(LabelElement,text.underlineObj), "-1"},
     { "-width", TK_OPTION_INT,
 	Tk_Offset(LabelElement,text.widthObj), ""},
+    { "-anchor", TK_OPTION_ANCHOR,
+	Tk_Offset(LabelElement,text.anchorObj), "center"},
     { "-justify", TK_OPTION_JUSTIFY,
 	Tk_Offset(LabelElement,text.justifyObj), "left" },
     { "-wraplength", TK_OPTION_PIXELS,
@@ -677,7 +685,7 @@ static void LabelElementDraw(
     /*
      * Adjust overall parcel based on -anchor:
      */
-    Tk_GetAnchorFromObj(NULL, l->anchorObj, &anchor);
+    Tk_GetAnchorFromObj(NULL, l->text.anchorObj, &anchor);
     b = Ttk_AnchorBox(b, l->totalWidth, l->totalHeight, anchor);
 
     /*
