@@ -5,7 +5,7 @@
  *
  * Copyright (c) 2003 Joe English.  Freely redistributable.
  *
- * layout.c,v 1.68 2006/04/16 19:28:55 jenglish Exp
+ * layout.c,v 1.70 2006/09/12 22:02:18 jenglish Exp
  */
 
 #include <string.h>
@@ -32,6 +32,20 @@ int Ttk_BoxContains(Ttk_Box box, int x, int y)
 	&& box.y <= y && y < box.y + box.height;
 }
 
+Tcl_Obj *Ttk_NewBoxObj(Ttk_Box box)
+{
+    Tcl_Obj *result[4];
+
+    result[0] = Tcl_NewIntObj(box.x);
+    result[1] = Tcl_NewIntObj(box.y);
+    result[2] = Tcl_NewIntObj(box.width);
+    result[3] = Tcl_NewIntObj(box.height);
+
+    return Tcl_NewListObj(4, result);
+}
+
+
+
 /*
  * packTop, packBottom, packLeft, packRight --
  * 	Carve out a parcel of the specified height (resp width)
