@@ -1,4 +1,4 @@
-/* manager.c,v 1.9 2005/10/08 15:53:42 jenglish Exp
+/* manager.c,v 1.12 2006/03/23 18:42:43 jenglish Exp
  *
  * Copyright 2005, Joe English.  Freely redistributable.
  *
@@ -44,10 +44,16 @@
  *
  */
 
+/* ++ manager->flags bits:
+ */
 #define MGR_UPDATE_PENDING	0x1
 #define MGR_RESIZE_REQUIRED	0x2
 #define MGR_RELAYOUT_REQUIRED	0x4
 
+/* ++ slave->flags bits:
+ */
+#define SLAVE_MAPPED 		0x1	/* slave to be mapped when master is */
+
 static void ManagerIdleProc(void *);	/* forward */
 
 /* ++ ScheduleUpdate --
@@ -97,7 +103,12 @@ static void ManagerIdleProc(ClientData clientData)
 
     if (mgr->flags & MGR_RESIZE_REQUIRED) {
 	RecomputeSize(mgr);
-    } else if (mgr->flags & MGR_RELAYOUT_REQUIRED) {
+    } 
+    if (mgr->flags & MGR_RELAYOUT_REQUIRED) {
+	if (mgr->flags & MGR_UPDATE_PENDING) {
+	    /* RecomputeSize has scheduled another update; relayout later */
+	    return;
+	}
 	RecomputeLayout(mgr);
     }
 }
@@ -108,13 +119,33 @@ static void ManagerIdleProc(ClientData clientData)
 
 /* ++ ManagerEventHandler --
  * 	Recompute slave layout when master widget is resized.
+ * 	Keep the slave's map state in sync with the master's.
  */
 static const int ManagerEventMask = StructureNotifyMask;
 static void ManagerEventHandler(ClientData clientData, XEvent *eventPtr)
 {
     Ttk_Manager *mgr = clientData;
-    if (eventPtr->type == ConfigureNotify) {
-	RecomputeLayout(mgr);
+    int i;
+
+    switch (eventPtr->type) 
+    {
+	case ConfigureNotify:
+	    RecomputeLayout(mgr);
+	    break;
+	case MapNotify:
+	    for (i = 0; i < mgr->nSlaves; ++i) {
+		Ttk_Slave *slave = mgr->slaves[i];
+		if (slave->flags & SLAVE_MAPPED) {
+		    Tk_MapWindow(slave->slaveWindow);
+		}
+	    }
+	    break;
+	case UnmapNotify:
+	    for (i = 0; i < mgr->nSlaves; ++i) {
+		Ttk_Slave *slave = mgr->slaves[i];
+		Tk_UnmapWindow(slave->slaveWindow);
+	    }
+	    break;
     }
 }
 
@@ -144,6 +175,7 @@ static Ttk_Slave *CreateSlave(
 
     slave->slaveWindow = slaveWindow;
     slave->manager = mgr;
+    slave->flags = 0;
     slave->slaveData = ckalloc(mgr->managerSpec->slaveSize);
     memset(slave->slaveData, 0, mgr->managerSpec->slaveSize);
 
@@ -320,7 +352,7 @@ int Ttk_AddSlave(
 
     /* Sanity-checks:
      */
-    if (!Maintainable(interp, slaveWindow, mgr->masterWindow)) {
+    if (!Ttk_Maintainable(interp, slaveWindow, mgr->masterWindow)) {
 	return TCL_ERROR;
     }
     if (Ttk_SlaveIndex(mgr, slaveWindow) >= 0) {
@@ -389,9 +421,12 @@ void Ttk_ForgetSlave(Ttk_Manager *mgr, int slaveIndex)
 void Ttk_PlaceSlave(
     Ttk_Manager *mgr, int slaveIndex, int x, int y, int width, int height)
 {
-    Tk_Window slaveWindow = mgr->slaves[slaveIndex]->slaveWindow;
-    Tk_MaintainGeometry(slaveWindow, mgr->masterWindow, x, y, width, height);
-    Tk_MapWindow(slaveWindow);
+    Ttk_Slave *slave = mgr->slaves[slaveIndex];
+    Tk_MaintainGeometry(slave->slaveWindow,mgr->masterWindow,x,y,width,height);
+    slave->flags |= SLAVE_MAPPED;
+    if (Tk_IsMapped(mgr->masterWindow)) {
+	Tk_MapWindow(slave->slaveWindow);
+    }
 }
 
 /* ++ Ttk_UnmapSlave --
@@ -399,12 +434,13 @@ void Ttk_PlaceSlave(
  */
 void Ttk_UnmapSlave(Ttk_Manager *mgr, int slaveIndex)
 {
-    Tk_Window slaveWindow = mgr->slaves[slaveIndex]->slaveWindow;
-    Tk_UnmaintainGeometry(slaveWindow, mgr->masterWindow);
+    Ttk_Slave *slave = mgr->slaves[slaveIndex];
+    Tk_UnmaintainGeometry(slave->slaveWindow, mgr->masterWindow);
+    slave->flags &= ~SLAVE_MAPPED;
     /* Contrary to documentation, Tk_UnmaintainGeometry doesn't always 
      * unmap the slave:
      */
-    Tk_UnmapWindow(slaveWindow);
+    Tk_UnmapWindow(slave->slaveWindow);
 }
 
 /* LayoutChanged, SizeChanged --
@@ -530,48 +566,18 @@ void Ttk_ReorderSlave(Ttk_Manager *mgr, int fromIndex, int toIndex)
     ScheduleUpdate(mgr, MGR_RESIZE_REQUIRED);
 }
 
-/* ++ Manageable --
- * 	Returns 1 if slave can be managed by manager;
- * 	otherwise returns 0 and leaves an error message in interp.
- *
- * Conditions:
- * 	+ slave must be a direct child of master;
- * 	+ slave must not be a top-level window.
- *
- * See also:
- * 	Maintainable().
- */
-int Manageable(Tcl_Interp *interp, Tk_Window slave, Tk_Window master)
-{
-    if (Tk_Parent(slave) != master) {
-	Tcl_AppendResult(interp,
-	    Tk_PathName(slave), " is not a child of ",
-	    Tk_PathName(master),
-	    NULL);
-	return 0;
-    }
-
-    if (Tk_IsTopLevel(slave)) {
-	Tcl_AppendResult(interp, "can't add ", Tk_PathName(slave),
-	    ": toplevel window",
-	    NULL);
-	return 0;
-    }
-
-    return 1;
-}
-
-/*
- * Maintainable(interp, slave, master) --
+/* ++ Ttk_Maintainable(interp, slave, master) --
  * 	Utility routine.  Verifies that 'master' may be used to maintain
- *	the geometry of 'slave' (via Tk_MaintainGeometry):
+ *	the geometry of 'slave' via Tk_MaintainGeometry:
  * 
  * 	+ 'master' is either 'slave's parent -OR-
  * 	+ 'master is a descendant of 'slave's parent.
  * 	+ 'slave' is not a toplevel window
  * 	+ 'slave' belongs to the same toplevel as 'master'
+ *
+ * Returns: 1 if OK; otherwise 0, leaving an error message in 'interp'.
  */
-int Maintainable(Tcl_Interp *interp, Tk_Window slave, Tk_Window master)
+int Ttk_Maintainable(Tcl_Interp *interp, Tk_Window slave, Tk_Window master)
 {
     Tk_Window ancestor = master, parent = Tk_Parent(slave), sibling = NULL;
 
