@@ -1,4 +1,4 @@
-/* manager.h,v 1.6 2005/08/19 21:54:10 jenglish Exp
+/* manager.h,v 1.8 2006/03/23 18:42:43 jenglish Exp
  *
  * Copyright (c) 2005, Joe English.  Freely redistributable.
  *
@@ -49,9 +49,10 @@ extern void Ttk_LostSlaveProc(ClientData, Tk_Window slave);
 
 struct TtkSlave_
 {
-    Tk_Window 	slaveWindow;
-    Ttk_Manager	*manager;
-    void 	*slaveData;
+    Tk_Window 		slaveWindow;
+    Ttk_Manager 	*manager;
+    void 		*slaveData;
+    unsigned		flags;			/* private; see manager.c */
 };
 
 struct TtkManager_
@@ -92,6 +93,10 @@ extern void Ttk_PlaceSlave(
 extern void Ttk_UnmapSlave(Ttk_Manager *, int slaveIndex);
     /* Unmap the slave */
 
+extern void Ttk_ManagerSizeChanged(Ttk_Manager *);
+extern void Ttk_ManagerLayoutChanged(Ttk_Manager *);
+    /* Notify manager that size (resp. layout) needs to be recomputed */
+
 /* Utilities:
  */
 extern int Ttk_SlaveIndex(Ttk_Manager *, Tk_Window);
@@ -111,13 +116,7 @@ extern void *Ttk_SlaveData(Ttk_Manager *, int slaveIndex);
 extern Tk_Window Ttk_SlaveWindow(Ttk_Manager *, int slaveIndex);
     /* Returns: slave window */
 
-extern void Ttk_ManagerLayoutChanged(Ttk_Manager *);
-extern void Ttk_ManagerSizeChanged(Ttk_Manager *);
-
-extern int Manageable(Tcl_Interp *interp, Tk_Window slave, Tk_Window master);
-    /* Returns: 1 if mgr can manage slave; otherwise 0 and leaves error msg */
-
-extern int Maintainable(Tcl_Interp *interp, Tk_Window slave, Tk_Window master);
-    /* Returns: 1 if Tk_MaintainGeometry is valid */
+extern int Ttk_Maintainable(Tcl_Interp *, Tk_Window slave, Tk_Window master);
+    /* Returns: 1 if master can manage slave; 0 otherwise leaving error msg */
 
 #endif
