@@ -1,7 +1,7 @@
-/* notebook.c,v 1.85 2006/05/06 17:59:28 jenglish Exp
+/* notebook.c,v 1.89 2006/09/20 14:39:56 jenglish Exp
  * Copyright (c) 2004, Joe English
  *
- * NOTE-ACTIVE: activeTabIndex is not always correct (it's  
+ * NOTE-ACTIVE: activeTabIndex is not always correct (it's
  * more trouble than it's worth to track this 100%)
  */
 
@@ -132,13 +132,13 @@ static Tk_OptionSpec NotebookOptionSpecs[] =
 };
 
 /* Notebook style options:
- * <<NOTE-POSITIONSPEC>> -- this isn't fully supported yet, just enough
- * to make (Jaguar-style) Aqua tab views work.
  */
-typedef struct 
+typedef struct
 {
     Ttk_PositionSpec	tabPosition;	/* Where to place tabs */
     Ttk_Padding 	tabMargins;	/* Margins around tab row */
+    Ttk_PositionSpec 	tabPlacement;	/* How to pack tabs within tab row */
+    Ttk_Orient		tabOrient;	/* ... */
     int 		minTabWidth;	/* Minimum tab width */
     Ttk_Padding 	padding;	/* External padding */
 } NotebookStyle;
@@ -152,6 +152,22 @@ static void NotebookStyleOptions(Notebook *nb, NotebookStyle *nbstyle)
 	TtkGetLabelAnchorFromObj(NULL, objPtr, &nbstyle->tabPosition);
     }
 
+    /* compute tabPlacement and tabOrient as function of tabPosition:
+     */
+    if (nbstyle->tabPosition & TTK_PACK_LEFT) {
+	nbstyle->tabPlacement = TTK_PACK_TOP | TTK_STICK_E;
+	nbstyle->tabOrient = TTK_ORIENT_VERTICAL;
+    } else if (nbstyle->tabPosition & TTK_PACK_RIGHT) {
+	nbstyle->tabPlacement = TTK_PACK_TOP | TTK_STICK_W;
+	nbstyle->tabOrient = TTK_ORIENT_VERTICAL;
+    } else if (nbstyle->tabPosition & TTK_PACK_BOTTOM) {
+	nbstyle->tabPlacement = TTK_PACK_LEFT | TTK_STICK_N;
+	nbstyle->tabOrient = TTK_ORIENT_HORIZONTAL;
+    } else { /* Assume TTK_PACK_TOP */
+	nbstyle->tabPlacement = TTK_PACK_LEFT | TTK_STICK_S;
+	nbstyle->tabOrient = TTK_ORIENT_HORIZONTAL;
+    }
+
     nbstyle->tabMargins = Ttk_UniformPadding(0);
     if ((objPtr = Ttk_QueryOption(nb->core.layout, "-tabmargins", 0)) != 0) {
 	Ttk_GetBorderFromObj(NULL, objPtr, &nbstyle->tabMargins);
@@ -182,7 +198,7 @@ static int IdentifyTab(Notebook *nb, int x, int y)
     int index;
     for (index = 0; index < Ttk_NumberSlaves(nb->notebook.mgr); ++index) {
 	Tab *tab = Ttk_SlaveData(nb->notebook.mgr,index);
-	if (	tab->state != TAB_STATE_HIDDEN 
+	if (	tab->state != TAB_STATE_HIDDEN
 	     && Ttk_BoxContains(tab->parcel, x,y))
 	{
 	    return index;
@@ -207,7 +223,7 @@ static void ActivateTab(Notebook *nb, int index)
  * TabState --
  * 	Return the state of the specified tab, based on
  * 	notebook state, currentIndex, activeIndex, and user-specified tab state.
- *	The USER1 bit is set for the leftmost tab, and USER2 
+ *	The USER1 bit is set for the leftmost tab, and USER2
  * 	is set for the rightmost tab.
  */
 static Ttk_State TabState(Notebook *nb, int index)
@@ -242,15 +258,18 @@ static Ttk_State TabState(Notebook *nb, int index)
  */
 
 /* TabrowSize --
- *	Compute max height and total width of all tabs.
+ *	Compute max height and total width of all tabs (horizontal layouts)
+ *	or total height and max width (vertical layouts).
  *
  * Side effects:
  * 	Sets width and height fields for all tabs.
- * 
+ *
  * Notes:
- * 	Hidden tabs are included in the height computation, but not width.
+ * 	Hidden tabs are included in the perpendicular computation
+ * 	(max height/width) but not parallel (total width/height).
  */
-static void TabrowSize(Notebook *nb, int *widthPtr, int *heightPtr)
+static void TabrowSize(
+    Notebook *nb, Ttk_Orient orient, int *widthPtr, int *heightPtr)
 {
     Ttk_Layout tabLayout = nb->notebook.tabLayout;
     int tabrowWidth = 0, tabrowHeight = 0;
@@ -263,9 +282,12 @@ static void TabrowSize(Notebook *nb, int *widthPtr, int *heightPtr)
 	Ttk_RebindSublayout(tabLayout, tab);
 	Ttk_LayoutSize(tabLayout,tabState,&tab->width,&tab->height);
 
-	tabrowHeight = MAX(tabrowHeight, tab->height);
-	if (tab->state != TAB_STATE_HIDDEN) {
-	    tabrowWidth += tab->width;
+	if (orient == TTK_ORIENT_HORIZONTAL) {
+	    tabrowHeight = MAX(tabrowHeight, tab->height);
+	    if (tab->state != TAB_STATE_HIDDEN) { tabrowWidth += tab->width; }
+	} else {
+	    tabrowWidth = MAX(tabrowWidth, tab->width);
+	    if (tab->state != TAB_STATE_HIDDEN) { tabrowHeight += tab->height; }
 	}
     }
 
@@ -273,7 +295,7 @@ static void TabrowSize(Notebook *nb, int *widthPtr, int *heightPtr)
     *heightPtr = tabrowHeight;
 }
 
-/* NotebookSize -- GM and widget size hook. 
+/* NotebookSize -- GM and widget size hook.
  *
  * Total height is tab height + client area height + pane internal padding
  * Total width is max(client width, tab width) + pane internal padding
@@ -299,9 +321,9 @@ static int NotebookSize(void *clientData, int *widthPtr, int *heightPtr)
     for (i = 0; i < Ttk_NumberSlaves(nb->notebook.mgr); ++i) {
 	Tk_Window slaveWindow = Ttk_SlaveWindow(nb->notebook.mgr, i);
 	Tab *tab = Ttk_SlaveData(nb->notebook.mgr, i);
-	int slaveWidth 
+	int slaveWidth
 	    = Tk_ReqWidth(slaveWindow) + Ttk_PaddingWidth(tab->padding);
-	int slaveHeight 
+	int slaveHeight
 	    = Tk_ReqHeight(slaveWindow) + Ttk_PaddingHeight(tab->padding);
 
 	clientWidth = MAX(clientWidth, slaveWidth);
@@ -319,7 +341,7 @@ static int NotebookSize(void *clientData, int *widthPtr, int *heightPtr)
 
     /* Tab row:
      */
-    TabrowSize(nb, &tabrowWidth, &tabrowHeight);
+    TabrowSize(nb, nbstyle.tabOrient, &tabrowWidth, &tabrowHeight);
     tabrowHeight += Ttk_PaddingHeight(nbstyle.tabMargins);
     tabrowWidth += Ttk_PaddingWidth(nbstyle.tabMargins);
 
@@ -346,7 +368,7 @@ static int NotebookSize(void *clientData, int *widthPtr, int *heightPtr)
  *	If the notebook is not wide enough to display all tabs,
  *	attempt to decrease tab widths to fit.
  *
- *	All tabs are shrunk by an equal amount, but will not be made 
+ *	All tabs are shrunk by an equal amount, but will not be made
  *	smaller than the minimum width.  (If all the tabs still do
  *	not fit in the available space, the rightmost tabs are truncated).
  *
@@ -355,7 +377,10 @@ static int NotebookSize(void *clientData, int *widthPtr, int *heightPtr)
  *	by one pixel will cause at most one tab to shrink by one pixel;
  *	this means that tabs resize "smoothly" when the window shrinks
  *	and grows.
+ *
+ * @@@ <<NOTE-TABPOSITION>> bug: only works for horizontal orientations
  */
+
 static void SqueezeTabs(
     Notebook *nb, int desiredWidth, int availableWidth, int minTabWidth)
 {
@@ -377,7 +402,8 @@ static void SqueezeTabs(
 /* PlaceTabs --
  * 	Compute all tab parcels.
  */
-static void PlaceTabs(Notebook *nb, Ttk_Box tabrowBox)
+static void PlaceTabs(
+    Notebook *nb, Ttk_Box tabrowBox, Ttk_PositionSpec tabPlacement)
 {
     Ttk_Layout tabLayout = nb->notebook.tabLayout;
     int nTabs = Ttk_NumberSlaves(nb->notebook.mgr);
@@ -395,10 +421,10 @@ static void PlaceTabs(Notebook *nb, Ttk_Box tabrowBox)
 		Ttk_GetBorderFromObj(NULL, expandObj, &expand);
 	    }
 
-	    tab->parcel = 
+	    tab->parcel =
 		Ttk_ExpandBox(
-		    Ttk_PlaceBox(&tabrowBox,
-			tab->width, tab->height, TTK_SIDE_LEFT, TTK_STICK_S),
+		    Ttk_PositionBox(&tabrowBox,
+			tab->width, tab->height, tabPlacement),
 		    expand);
 	}
     }
@@ -432,7 +458,7 @@ static void NotebookDoLayout(void *recordPtr)
 
     /* Place tabs:
      */
-    TabrowSize(nb, &tabrowWidth, &tabrowHeight);
+    TabrowSize(nb, nbstyle.tabOrient, &tabrowWidth, &tabrowHeight);
     tabrowBox = Ttk_PadBox(
 		    Ttk_PositionBox(&cavity,
 			tabrowWidth + Ttk_PaddingWidth(nbstyle.tabMargins),
@@ -443,7 +469,7 @@ static void NotebookDoLayout(void *recordPtr)
     if (tabrowWidth > tabrowBox.width) {
 	SqueezeTabs(nb, tabrowWidth, tabrowBox.width, nbstyle.minTabWidth);
     }
-    PlaceTabs(nb, tabrowBox);
+    PlaceTabs(nb, tabrowBox, nbstyle.tabPlacement);
 
     /* Layout for client area frame:
      */
@@ -467,7 +493,7 @@ static void NotebookPlaceSlave(Notebook *nb, int slaveIndex)
 {
     Tab *tab = Ttk_SlaveData(nb->notebook.mgr, slaveIndex);
     Tk_Window slaveWindow = Ttk_SlaveWindow(nb->notebook.mgr, slaveIndex);
-    Ttk_Box slaveBox = 
+    Ttk_Box slaveBox =
 	Ttk_StickBox(Ttk_PadBox(nb->notebook.clientArea, tab->padding),
 	    Tk_ReqWidth(slaveWindow), Tk_ReqHeight(slaveWindow),tab->sticky);
 
@@ -505,7 +531,7 @@ static void SelectTab(Notebook *nb, int index)
 	return;
     }
 
-    /* Unhide the tab if it is currently hidden and being selected. 
+    /* Unhide the tab if it is currently hidden and being selected.
      */
     if (tab->state == TAB_STATE_HIDDEN) {
 	tab->state = TAB_STATE_NORMAL;
@@ -532,7 +558,7 @@ static int NextTab(Notebook *nb, int index)
 {
     int nTabs = Ttk_NumberSlaves(nb->notebook.mgr);
     int nextIndex;
-   
+
     /* Scan forward for following usable tab:
      */
     for (nextIndex = index + 1; nextIndex < nTabs; ++nextIndex) {
@@ -557,11 +583,11 @@ static int NextTab(Notebook *nb, int index)
 }
 
 /* SelectNearestTab --
- * 	Handles the case where the current tab is forgotten, hidden, 
+ * 	Handles the case where the current tab is forgotten, hidden,
  * 	or destroyed. Select the next available tab; or, if there is none,
  * 	leaves all tabs unselected.
  */
-static void SelectNearestTab(Notebook *nb) 
+static void SelectNearestTab(Notebook *nb)
 {
     int nextIndex = NextTab(nb, nb->notebook.currentIndex);
 
@@ -617,7 +643,7 @@ static int TabConfigured(
     if (Ttk_GetStickyFromObj(interp, tab->stickyObj, &sticky) != TCL_OK) {
 	return TCL_ERROR;
     }
-    if (Ttk_GetPaddingFromObj(interp,tkwin,tab->paddingObj,&tab->padding) 
+    if (Ttk_GetPaddingFromObj(interp,tkwin,tab->paddingObj,&tab->padding)
 	!= TCL_OK)
     {
 	return TCL_ERROR;
@@ -627,7 +653,7 @@ static int TabConfigured(
     return TCL_OK;
 }
 
-static Ttk_ManagerSpec NotebookManagerSpec = 
+static Ttk_ManagerSpec NotebookManagerSpec =
 {
     { "notebook", Ttk_GeometryRequestProc, Ttk_LostSlaveProc },
     PaneOptionSpecs, sizeof(Tab),
@@ -646,17 +672,17 @@ static Ttk_ManagerSpec NotebookManagerSpec =
 /* NotebookEventHandler --
  * 	Tracks the active tab.
  */
-static const int NotebookEventMask 
-    = StructureNotifyMask 
+static const int NotebookEventMask
+    = StructureNotifyMask
     | PointerMotionMask
-    | LeaveWindowMask 
+    | LeaveWindowMask
     ;
 static void NotebookEventHandler(ClientData clientData, XEvent *eventPtr)
 {
     Notebook *nb = clientData;
 
     if (eventPtr->type == DestroyNotify) { /* Remove self */
-	Tk_DeleteEventHandler(nb->core.tkwin, 
+	Tk_DeleteEventHandler(nb->core.tkwin,
 	    NotebookEventMask, NotebookEventHandler, clientData);
     } else if (eventPtr->type == MotionNotify) {
 	int index = IdentifyTab(nb, eventPtr->xmotion.x, eventPtr->xmotion.y);
@@ -670,7 +696,7 @@ static void NotebookEventHandler(ClientData clientData, XEvent *eventPtr)
  * +++ Utilities.
  */
 
-/* GetTabIndex --
+/* FindTabIndex --
  *	Find the index of the specified tab.
  *	Tab identifiers are one of:
  *
@@ -679,15 +705,15 @@ static void NotebookEventHandler(ClientData clientData, XEvent *eventPtr)
  *	+ numeric indices [0..nTabs],
  *	+ slave window names
  *
- *	Returns: TCL_OK or TCL_ERROR.
  *	Stores index of specified tab in *index_rtn, -1 if not found.
- *	Leaves an error message in interp in case of error.
+ *
+ *	Returns TCL_ERROR and leaves an error message in interp->result
+ *	if the tab identifier was incorrect.
+ *
+ *	See also: GetTabIndex.
  */
-static int GetTabIndex(
-    Tcl_Interp *interp,		/* Where to leave error messages */
-    Notebook *nb,		/* Notebook widget record */
-    Tcl_Obj *objPtr,		/* Tab name to look up */
-    int *index_rtn)
+static int FindTabIndex(
+    Tcl_Interp *interp, Notebook *nb, Tcl_Obj *objPtr, int *index_rtn)
 {
     const char *string = Tcl_GetString(objPtr);
     int x, y;
@@ -711,7 +737,7 @@ static int GetTabIndex(
     /* ... or integer index or slave window name:
      */
     if (Ttk_GetSlaveFromObj(
-	    interp, nb->notebook.mgr, objPtr, index_rtn) != NULL) 
+	    interp, nb->notebook.mgr, objPtr, index_rtn) != NULL)
     {
 	return TCL_OK;
     }
@@ -721,6 +747,26 @@ static int GetTabIndex(
     return TCL_ERROR;
 }
 
+/* GetTabIndex --
+ * 	Get the index of an existing tab.
+ * 	Tab identifiers are as per FindTabIndex.
+ * 	Returns TCL_ERROR if the tab does not exist.
+ */
+static int GetTabIndex(
+    Tcl_Interp *interp, Notebook *nb, Tcl_Obj *objPtr, int *index_rtn)
+{
+    int status = FindTabIndex(interp, nb, objPtr, index_rtn);
+
+    if (status == TCL_OK && *index_rtn < 0) {
+	Tcl_ResetResult(interp);
+	Tcl_AppendResult(interp,
+	    "tab '", Tcl_GetString(objPtr), "' not found",
+	    NULL);
+	status = TCL_ERROR;
+    }
+    return status;
+}
+
 /*------------------------------------------------------------------------
  * +++ Widget command routines.
  */
@@ -790,14 +836,14 @@ static int NotebookInsertCommand(
     if (!Ttk_GetSlaveFromObj(interp, nb->notebook.mgr, objv[3], &srcIndex)) {
 	/* Try adding new slave:
 	 */
-	Tk_Window slaveWindow = 
+	Tk_Window slaveWindow =
 	    Tk_NameToWindow(interp,Tcl_GetString(objv[3]),nb->core.tkwin);
 	if (!slaveWindow) {
 	    return TCL_ERROR;
 	}
 
 	if (Ttk_AddSlave(interp, nb->notebook.mgr, slaveWindow,
-		destIndex, objc - 4, objv + 4) != TCL_OK) 
+		destIndex, objc - 4, objv + 4) != TCL_OK)
 	{
 	    return TCL_ERROR;
 	}
@@ -858,6 +904,47 @@ static int NotebookForgetCommand(
     return TCL_OK;
 }
 
+/* $nb identify $x $y --
+ * 	Returns name of tab element at $x,$y; empty string if none.
+ */
+static int NotebookIdentifyCommand(
+    Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
+{
+    Notebook *nb = recordPtr;
+    Ttk_LayoutNode *node = NULL;
+    int x, y, tabIndex;
+
+    if (objc != 4) {
+	Tcl_WrongNumArgs(interp, 2, objv, "x y");
+	return TCL_ERROR;
+    }
+
+    if (   Tcl_GetIntFromObj(interp, objv[2], &x) != TCL_OK
+	|| Tcl_GetIntFromObj(interp, objv[3], &y) != TCL_OK)
+    {
+	return TCL_ERROR;
+    }
+
+    tabIndex = IdentifyTab(nb, x, y);
+    if (tabIndex >= 0) {
+	Tab *tab = Ttk_SlaveData(nb->notebook.mgr, tabIndex);
+	Ttk_State state = TabState(nb, tabIndex);
+	Ttk_Layout tabLayout = nb->notebook.tabLayout;
+
+	Ttk_RebindSublayout(tabLayout, tab);
+	Ttk_PlaceLayout(tabLayout, state, tab->parcel);
+
+	node = Ttk_LayoutIdentify(tabLayout, x, y); 
+    }
+
+    if (node) {
+	const char *elementName = Ttk_LayoutNodeName(node);
+	Tcl_SetObjResult(interp,Tcl_NewStringObj(elementName,-1));
+    }
+
+    return TCL_OK;
+}
+
 /* $nb index $item --
  * 	Returns the integer index of the tab specified by $item,
  * 	the empty string if $item does not identify a tab.
@@ -882,7 +969,7 @@ static int NotebookIndexCommand(
 	return TCL_OK;
     }
 
-    status = GetTabIndex(interp, nb, objv[2], &index);
+    status = FindTabIndex(interp, nb, objv[2], &index);
     if (status == TCL_OK && index >= 0) {
 	Tcl_SetObjResult(interp, Tcl_NewIntObj(index));
     }
@@ -967,7 +1054,7 @@ static int NotebookTabCommand(
     tab = Ttk_SlaveData(mgr, index);
 
     if (objc == 3) {
-	return EnumerateOptions(interp, tab, 
+	return EnumerateOptions(interp, tab,
 	    PaneOptionSpecs, nb->notebook.paneOptionTable, nb->core.tkwin);
     } else if (objc == 4) {
 	return GetOptionValue(interp, tab, objv[3],
@@ -978,7 +1065,7 @@ static int NotebookTabCommand(
 	return TCL_ERROR;
     }
 
-    /* If the current tab has become disabled or hidden, 
+    /* If the current tab has become disabled or hidden,
      * select the next nondisabled, unhidden one:
      */
     if (index == nb->notebook.currentIndex && tab->state != TAB_STATE_NORMAL) {
@@ -997,6 +1084,7 @@ static WidgetCommandSpec NotebookCommands[] =
     { "configure",	WidgetConfigureCommand },
     { "cget",		WidgetCgetCommand },
     { "forget",		NotebookForgetCommand },
+    { "identify",	NotebookIdentifyCommand },
     { "index",		NotebookIndexCommand },
     { "insert",  	NotebookInsertCommand },
     { "instate",	WidgetInstateCommand },
@@ -1118,7 +1206,7 @@ static void NotebookDisplay(void *clientData, Drawable d)
     Ttk_DrawLayout(nb->core.layout, nb->core.state, d);
 
     /* Draw tabs from left to right, but draw the current tab last
-     * so it will overwrite its neighbors. 
+     * so it will overwrite its neighbors.
      */
     for (index = 0; index < nb->notebook.mgr->nSlaves; ++index) {
 	if (index != nb->notebook.currentIndex) {
@@ -1156,8 +1244,8 @@ TTK_END_LAYOUT
 
 TTK_BEGIN_LAYOUT(TabLayout)
     TTK_GROUP("Notebook.tab", TTK_FILL_BOTH,
-	TTK_GROUP("Notebook.padding", TTK_PACK_TOP|TTK_FILL_BOTH, 
-	    TTK_GROUP("Notebook.focus", TTK_PACK_TOP|TTK_FILL_BOTH, 
+	TTK_GROUP("Notebook.padding", TTK_PACK_TOP|TTK_FILL_BOTH,
+	    TTK_GROUP("Notebook.focus", TTK_PACK_TOP|TTK_FILL_BOTH,
 		TTK_NODE("Notebook.label", TTK_PACK_TOP))))
 TTK_END_LAYOUT
 
