@@ -1,4 +1,4 @@
-/* progress.c,v 1.13 2006/01/31 07:30:24 jenglish Exp
+/* progress.c,v 1.14 2006/08/04 23:22:50 patthoyts Exp
  *
  * Copyright (c) Joe English, Pat Thoyts, Michael Kirkham
  *
@@ -490,6 +490,7 @@ static WidgetCommandSpec ProgressbarCommands[] =
 {
     { "configure",	WidgetConfigureCommand },
     { "cget",		WidgetCgetCommand },
+    { "identify",	WidgetIdentifyCommand },
     { "instate",	WidgetInstateCommand },
     { "start", 		ProgressbarStartCommand },
     { "state",  	WidgetStateCommand },
