@@ -1,4 +1,4 @@
-/* scrollbar.c,v 1.44 2005/11/10 18:18:25 jenglish Exp
+/* scrollbar.c,v 1.46 2006/09/26 12:45:10 jenglish Exp
  * Copyright (c) 2003, Joe English
  * Tile widget set: scrollbar widget implementation.
  */
@@ -186,19 +186,17 @@ ScrollbarGetCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     Scrollbar *scrollbar = recordPtr;
-    Tcl_Obj *result;
+    Tcl_Obj *result[2];
 
     if (objc != 2) {
 	Tcl_WrongNumArgs(interp, 2, objv, "");
 	return TCL_ERROR;
     }
 
-    result = Tcl_NewListObj(0,0);
-    Tcl_ListObjAppendElement(interp, result, 
-	    Tcl_NewDoubleObj(scrollbar->scrollbar.first));
-    Tcl_ListObjAppendElement(interp, result, 
-	    Tcl_NewDoubleObj(scrollbar->scrollbar.last));
-    Tcl_SetObjResult(interp, result);
+    result[0] = Tcl_NewDoubleObj(scrollbar->scrollbar.first);
+    result[1] = Tcl_NewDoubleObj(scrollbar->scrollbar.last);
+    Tcl_SetObjResult(interp, Tcl_NewListObj(2, result));
+
     return TCL_OK;
 }
 
@@ -228,12 +226,14 @@ ScrollbarDeltaCommand(
     delta = 0.0;
     if (sb->scrollbar.orient == TTK_ORIENT_VERTICAL) {
 	int size = sb->scrollbar.troughBox.height - sb->scrollbar.minSize;
-	if (size > 0)
+	if (size > 0) {
 	    delta = (double)dy / (double)size;
+	}
     } else {
 	int size = sb->scrollbar.troughBox.width - sb->scrollbar.minSize;
-	if (size > 0)
+	if (size > 0) {
 	    delta = (double)dx / (double)size;
+	}
     }
 
     Tcl_SetObjResult(interp, Tcl_NewDoubleObj(delta));
@@ -267,11 +267,13 @@ ScrollbarFractionCommand(
 
     fraction = 0.0;
     if (sb->scrollbar.orient == TTK_ORIENT_VERTICAL) {
-	if (b.height != 0)
+	if (b.height > minSize) {
 	    fraction = (double)(y - b.y) / (double)(b.height - minSize);
+	}
     } else {
-	if (sb->scrollbar.troughBox.width != 0)
+	if (b.width > minSize) {
 	    fraction = (double)(x - b.x) / (double)(b.width - minSize);
+	}
     }
 
     Tcl_SetObjResult(interp, Tcl_NewDoubleObj(fraction));
