@@ -1,8 +1,8 @@
-/* separator.c,v 1.5 2005/08/17 23:05:41 jenglish Exp
+/* separator.c,v 1.7 2006/08/05 05:59:50 jenglish Exp
  *
  * Copyright (c) 2004, Joe English
  *
- * Tile package: separator widget.
+ * Tile package: separator and sizegrip widgets.
  */
 
 #include <tk.h>
@@ -10,8 +10,7 @@
 #include "tkTheme.h"
 #include "widget.h"
 
-/*
- * Widget record:
+/* +++ Separator widget record:
  */
 typedef struct
 {
@@ -54,6 +53,7 @@ static WidgetCommandSpec SeparatorCommands[] =
 {
     { "configure",	WidgetConfigureCommand },
     { "cget",		WidgetCgetCommand },
+    { "identify",	WidgetIdentifyCommand },
     { "instate",	WidgetInstateCommand },
     { "state",  	WidgetStateCommand },
     { NULL, NULL }
@@ -78,4 +78,34 @@ WidgetSpec SeparatorWidgetSpec =
     WidgetDisplay		/* displayProc */
 };
 
+/* +++ Sizegrip widget:
+ * 	Has no options or methods other than the standard ones.
+ */
+
+static WidgetCommandSpec SizegripCommands[] =
+{
+    { "configure",	WidgetConfigureCommand },
+    { "cget",		WidgetCgetCommand },
+    { "identify",	WidgetIdentifyCommand },
+    { "instate",	WidgetInstateCommand },
+    { "state",  	WidgetStateCommand },
+    { NULL, NULL }
+};
+
+WidgetSpec SizegripWidgetSpec =
+{
+    "TSizegrip",		/* className */
+    sizeof(WidgetCore),		/* recordSize */
+    CoreOptionSpecs, 		/* optionSpecs */
+    SizegripCommands,		/* subcommands */
+    NullInitialize,		/* initializeProc */
+    NullCleanup,		/* cleanupProc */
+    CoreConfigure,		/* configureProc */
+    NullPostConfigure,		/* postConfigureProc */
+    WidgetGetLayout, 		/* getLayoutProc */
+    WidgetSize, 		/* sizeProc */
+    WidgetDoLayout,		/* layoutProc */
+    WidgetDisplay		/* displayProc */
+};
+
 /*EOF*/
