@@ -2,7 +2,7 @@
  *
  * Minimal sample tile widget.
  *
- * square.c,v 1.4 2005/08/17 23:05:41 jenglish Exp
+ * square.c,v 1.6 2006/10/05 00:05:43 patthoyts Exp
  */
 
 #include <tk.h>
@@ -41,6 +41,11 @@ typedef struct
 /*
  * Widget options.
  *
+ * This structure is the same as the option specification structure used
+ * for Tk widgets. For each option we provide the type, name and options
+ * database name and class name and the position in the structure and
+ * default values. At the bottom we bring in the standard widget option
+ * defined for all widgets.
  */
 
 static Tk_OptionSpec SquareOptionSpecs[] =
@@ -51,7 +56,7 @@ static Tk_OptionSpec SquareOptionSpecs[] =
      DEFAULT_BORDERWIDTH, Tk_Offset(Square,square.borderWidthObj), -1,
      0,0,GEOMETRY_CHANGED },
     {TK_OPTION_BORDER, "-foreground", "foreground", "Foreground",
-     "#eeaaaa", Tk_Offset(Square,square.foregroundObj),
+     DEFAULT_BACKGROUND, Tk_Offset(Square,square.foregroundObj),
      -1, 0, 0, 0},
     
     {TK_OPTION_PIXELS, "-width", "width", "Width",
@@ -74,22 +79,72 @@ static Tk_OptionSpec SquareOptionSpecs[] =
     WIDGET_INHERIT_OPTIONS(CoreOptionSpecs)
 };
 
+/*
+ * Almost all of the widget functionality is handled by the default Tile
+ * widget code and the contained element. The one thing that we must handle
+ * is the -anchor option which positions the square element within the parcel
+ * of space available for the widget.
+ * To do this we must find out the layout preferences for the square
+ * element and adjust its position within our region.
+ *
+ * Note that if we do not have a "square" elememt then just the default
+ * layout will be done. So if someone places a label element into the
+ * widget layout it will still be handled but the -anchor option will be
+ * passed onto the label element instead of handled here.
+ */
+
+static void
+SquareDoLayout(void *clientData)
+{
+    WidgetCore *corePtr = (WidgetCore *)clientData;
+    Ttk_Box winBox;
+    Ttk_LayoutNode *squareNode;
+
+    squareNode = Ttk_LayoutFindNode(corePtr->layout, "square");
+    winBox = Ttk_WinBox(corePtr->tkwin);
+    Ttk_PlaceLayout(corePtr->layout, corePtr->state, winBox);
+
+    /*
+     * Adjust the position of the square element within the widget according
+     * to the -anchor option.
+     */
+    
+    if (squareNode) {
+	Square *squarePtr = clientData;
+	Tk_Anchor anchor = TK_ANCHOR_CENTER;
+	Ttk_Box b;
+	
+	b = Ttk_LayoutNodeParcel(squareNode);
+	if (squarePtr->square.anchorObj != NULL)
+	    Tk_GetAnchorFromObj(NULL, squarePtr->square.anchorObj, &anchor);
+	b = Ttk_AnchorBox(winBox, b.width, b.height, anchor);
+	
+	Ttk_PlaceLayoutNode(corePtr->layout, squareNode, b);
+    }
+}
 
 /*
- * Widget commands:
+ * Widget commands. A widget is impelemented as an ensemble and the
+ * subcommands are listed here. Tile provides default implementations
+ * that are sufficient for our needs.
  */
+
 static WidgetCommandSpec SquareCommands[] =
 {
     { "configure",	WidgetConfigureCommand },
     { "cget",		WidgetCgetCommand },
+    { "identify",	WidgetIdentifyCommand },
     { "instate",	WidgetInstateCommand },
     { "state",  	WidgetStateCommand },
     { NULL, NULL }
 };
 
 /*
- * Widget specification: 
+ * The Widget specification structure holds all the implementation 
+ * information about this widget and this is what must be registered
+ * with Tk in the package initialization code (see bottom).
  */
+
 WidgetSpec SquareWidgetSpec =
 {
     "TSquare",			/* className */
@@ -102,7 +157,7 @@ WidgetSpec SquareWidgetSpec =
     NullPostConfigure,		/* postConfigureProc */
     WidgetGetLayout,		/* getLayoutProc */
     WidgetSize, 		/* sizeProc */
-    WidgetDoLayout,		/* layoutProc */
+    SquareDoLayout,		/* layoutProc */
     WidgetDisplay		/* displayProc */
 };
 
@@ -132,12 +187,18 @@ static Ttk_ElementOptionSpec SquareElementOptions[] =
     { "-borderwidth", TK_OPTION_PIXELS, Tk_Offset(SquareElement,borderWidthObj),
     	DEFAULT_BORDERWIDTH },
     { "-relief", TK_OPTION_RELIEF, Tk_Offset(SquareElement,reliefObj),
-    	"flat" },
+    	"raised" },
     { "-width",  TK_OPTION_PIXELS, Tk_Offset(SquareElement,widthObj), "20"},
     { "-height", TK_OPTION_PIXELS, Tk_Offset(SquareElement,heightObj), "20"},
     { NULL }
 };
 
+/*
+ * The element geometry function is called when the layout code wishes to
+ * find out how big this element wants to be. We must return our preferred
+ * size and padding information
+ */
+
 static void
 SquareElementGeometry(
     void *clientData, void *elementRecord,
@@ -152,6 +213,10 @@ SquareElementGeometry(
     Tk_GetPixelsFromObj(NULL, tkwin, square->heightObj, heightPtr);
 }
 
+/*
+ * Draw the element in the box provided.
+ */
+
 static void
 SquareElementDraw(void *clientData, void *elementRecord,
     Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
@@ -193,7 +258,7 @@ static Ttk_ElementSpec SquareElementSpec =
 TTK_BEGIN_LAYOUT(SquareLayout)
      TTK_NODE("Square.background", TTK_FILL_BOTH)
      TTK_GROUP("Square.padding", TTK_FILL_BOTH,
-	 TTK_NODE("Square.square", TTK_PACK_TOP))
+	 TTK_NODE("Square.square", 0))
 TTK_END_LAYOUT
 
 /* ---------------------------------------------------------------------- 
@@ -235,3 +300,4 @@ SquareWidget_Init(Tcl_Interp *interp)
 
     return TCL_OK;
 }
+
