@@ -78,17 +78,18 @@ Draw3DCircle(Tk_Window tkwin, Drawable d, int x, int y, int diameter, Tk_3DBorde
 	(unsigned)diameter, (unsigned)diameter, 0, 360*64);
     DrawRing(display, d, gcDark, x, y, 3, 7, 0); /* ring - 0 */
     DrawRing(display, d, gcBlack, x, y, 3, 5, 1); /* ring - 1 */
-    //DrawRing(display, d, gcDark,  x, y, 2, 5, 2); /* ring - 2 */
-    //DrawRing(display, d, gcBlack, x, y, 2, 3, 3); /* ring - 3 */
-    //DrawRing(display, d, gcDark,  x, y, 1, 3, 4); /* ring - 4 */
-    //DrawRing(display, d, gcBlack, x, y, 1, 1, 5); /* ring - 5 */
+#if 0
+    DrawRing(display, d, gcDark,  x, y, 2, 5, 2); /* ring - 2 */
+    DrawRing(display, d, gcBlack, x, y, 2, 3, 3); /* ring - 3 */
+    DrawRing(display, d, gcDark,  x, y, 1, 3, 4); /* ring - 4 */
+    DrawRing(display, d, gcBlack, x, y, 1, 1, 5); /* ring - 5 */
 
-    //XDrawArc(display, d, gcDark,  x, y, diameter, diameter, 30*64, 210*64 /*0, 360*64*/);
-    //XDrawArc(display, d, gcBlack, x+1, y+1, diameter-1, diameter-1, 30*64, 210*64);
-
-//    XDrawArc(display, d, gcLight, x+1, y+1, diameter-2, diameter-2, 30*64, 210*64);
-    //XDrawArc(display, d, gcDark, x-1, y-1, diameter-2, diameter-2, 230*64, 170*64);
+    XDrawArc(display, d, gcDark,  x, y, diameter, diameter, 30*64, 210*64 /*0, 360*64*/);
+    XDrawArc(display, d, gcBlack, x+1, y+1, diameter-1, diameter-1, 30*64, 210*64);
 
+    XDrawArc(display, d, gcLight, x+1, y+1, diameter-2, diameter-2, 30*64, 210*64);
+    XDrawArc(display, d, gcDark, x-1, y-1, diameter-2, diameter-2, 230*64, 170*64);
+#endif
     Tk_FreeGC(Tk_Display(tkwin), gcBlack);
 }
 #endif /*PAT_EXPERIMENTAL*/
@@ -525,7 +526,7 @@ static void PbarElementDraw(
 	b.x, b.y, b.width, b.height,
 	borderWidth, relief);
 
-    b = Ttk_PadBox(b, Ttk_UniformPadding(borderWidth));
+    b = Ttk_PadBox(b, Ttk_UniformPadding((short)borderWidth));
     PbarElementDrawStripes(clientData, elementRecord, tkwin, d, b, state);
 }
 
