@@ -1,4 +1,4 @@
-/* tile.c,v 1.69 2005/09/29 15:34:54 jenglish Exp
+/* tile.c,v 1.70 2006/08/05 05:59:50 jenglish Exp
  * Copyright (c) 2003, Joe English
  *
  * Tile package: initialization routine and miscellaneous utilities.
@@ -202,6 +202,7 @@ extern WidgetSpec MenubuttonWidgetSpec;
 extern WidgetSpec ScrollbarWidgetSpec;
 extern WidgetSpec ScaleWidgetSpec;
 extern WidgetSpec SeparatorWidgetSpec;
+extern WidgetSpec SizegripWidgetSpec;
 
 extern void Progressbar_Init(Tcl_Interp *);
 extern void Notebook_Init(Tcl_Interp *);
@@ -225,6 +226,7 @@ static void RegisterWidgets(Tcl_Interp *interp)
     RegisterWidget(interp, "ttk::scrollbar", &ScrollbarWidgetSpec);
     RegisterWidget(interp, "ttk::scale", &ScaleWidgetSpec);
     RegisterWidget(interp, "ttk::separator", &SeparatorWidgetSpec);
+    RegisterWidget(interp, "ttk::sizegrip", &SizegripWidgetSpec);
     Notebook_Init(interp);
     EntryWidget_Init(interp);
     Treeview_Init(interp);
