@@ -1,4 +1,4 @@
-/* tkElements.c,v 1.77 2005/12/12 20:18:19 jenglish Exp
+/* tkElements.c,v 1.79 2006/08/05 05:59:50 jenglish Exp
  *
  * Copyright (c) 2003, Joe English
  *
@@ -448,8 +448,57 @@ static Ttk_ElementSpec SeparatorElementSpec =
     GeneralSeparatorElementDraw
 };
 
-/*
- *----------------------------------------------------------------------
+/*----------------------------------------------------------------------
+ * +++ Sizegrip: lower-right corner grip handle for resizing window.
+ */
+
+typedef struct
+{
+    Tcl_Obj	*backgroundObj;
+} SizegripElement;
+
+static Ttk_ElementOptionSpec SizegripOptions[] = {
+    { "-background", TK_OPTION_BORDER,
+	Tk_Offset(SizegripElement,backgroundObj), DEFAULT_BACKGROUND },
+    {0,0,0,0}
+};
+
+static void SizegripSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+{
+    int gripCount = 3, gripSpace = 2, gripThickness = 3;
+    *widthPtr = *heightPtr = gripCount * (gripSpace + gripThickness);
+}
+
+static void SizegripDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, Ttk_State state)
+{
+    SizegripElement *grip = elementRecord;
+    int gripCount = 3, gripSpace = 2;
+    Tk_3DBorder border = Tk_Get3DBorderFromObj(tkwin, grip->backgroundObj);
+    GC lightGC = Tk_3DBorderGC(tkwin, border, TK_3D_LIGHT_GC);
+    GC darkGC = Tk_3DBorderGC(tkwin, border, TK_3D_DARK_GC);
+    int x1 = b.x + b.width-1, y1 = b.y + b.height-1, x2 = x1, y2 = y1;
+
+    while (gripCount--) {
+	x1 -= gripSpace; y2 -= gripSpace;
+	XDrawLine(Tk_Display(tkwin), d, darkGC,  x1,y1, x2,y2); --x1; --y2;
+	XDrawLine(Tk_Display(tkwin), d, darkGC,  x1,y1, x2,y2); --x1; --y2;
+	XDrawLine(Tk_Display(tkwin), d, lightGC, x1,y1, x2,y2); --x1; --y2;
+    }
+}
+
+static Ttk_ElementSpec SizegripElementSpec = {
+    TK_STYLE_VERSION_2,
+    sizeof(SizegripElement),
+    SizegripOptions,
+    SizegripSize,
+    SizegripDraw
+};
+
+/*----------------------------------------------------------------------
  * +++ Indicator element.
  *
  * Draws the on/off indicator for checkbuttons and radiobuttons.
@@ -1303,6 +1352,10 @@ TTK_BEGIN_LAYOUT(SeparatorLayout)
     TTK_NODE("Separator.separator", TTK_FILL_BOTH)
 TTK_END_LAYOUT
 
+TTK_BEGIN_LAYOUT(SizegripLayout)
+    TTK_NODE("Sizegrip.sizegrip", TTK_PACK_BOTTOM|TTK_STICK_S|TTK_STICK_E)
+TTK_END_LAYOUT
+
 /*----------------------------------------------------------------------
  * RegisterElements --
  *
@@ -1366,6 +1419,7 @@ void RegisterElements(Tcl_Interp *interp)
     Ttk_RegisterElement(interp, theme, "vseparator", 
 	    &VerticalSeparatorElementSpec, NULL);
 
+    Ttk_RegisterElement(interp, theme, "sizegrip", &SizegripElementSpec, NULL);
 
     Ttk_RegisterElement(interp, theme, "tab", &TabElementSpec, NULL);
     Ttk_RegisterElement(interp, theme, "client", &ClientElementSpec, NULL);
@@ -1387,6 +1441,7 @@ void RegisterElements(Tcl_Interp *interp)
     Ttk_RegisterLayout(theme, "Vertical.TScale", VerticalScaleLayout);
     Ttk_RegisterLayout(theme, "Horizontal.TScale", HorizontalScaleLayout);
     Ttk_RegisterLayout(theme, "TSeparator", SeparatorLayout);
+    Ttk_RegisterLayout(theme, "TSizegrip", SizegripLayout);
 }
 
 /*EOF*/
