@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * tkTheme.c,v 1.89 2006/04/06 19:34:11 jenglish Exp
+ * tkTheme.c,v 1.91 2006/09/20 23:57:41 jenglish Exp
  */
 
 #include <stdlib.h>
@@ -959,7 +959,9 @@ static int AllocateResource(
  * InitializeElementRecord --
  *
  * 	Fill in the element record based on the element's option table.
- * 	Resources are initialized from the dynamic state map if specified,
+ * 	Resources are initialized from:
+ * 	the corresponding widget option if present and non-NULL,
+ * 	otherwise the dynamic state map if specified,
  * 	otherwise from the corresponding widget resource if present,
  * 	otherwise the default value specified at registration time.
  *
@@ -1004,10 +1006,10 @@ int InitializeElementRecord(
 		(widgetRecord + optionMap[i]->objOffset);
 	}
 
-	if (dynamicSetting) {
-	    *dest = dynamicSetting;
-	} else if (widgetValue) {
+	if (widgetValue) {
 	    *dest = widgetValue;
+	} else if (dynamicSetting) {
+	    *dest = dynamicSetting;
 	} else {
 	    Tcl_Obj *styleDefault = LookupDefault(style, optionName);
 	    *dest = styleDefault ? styleDefault : elementDefault;
@@ -1034,22 +1036,12 @@ Tcl_Obj *Ttk_QueryStyle(
     void *recordPtr,		/* Widget record */
     Tk_OptionTable optionTable,	/* Option table describing widget record */
     const char *optionName,	/* Option name */
-    Ttk_State state)	/* Current state */
+    Ttk_State state) 		/* Current state */
 {
-    Tcl_Obj *stateMap = LookupStateMap(style, optionName);
+    Tcl_Obj *stateMap;
     const Tk_OptionSpec *optionSpec;
     Tcl_Obj *result;
 
-    /*
-     * Check dynamic settings:
-     */
-    if (stateMap) {
-	result = Ttk_StateMapLookup(NULL, stateMap, state);
-	if (result) {
-	    return result;
-	}
-    }
-
     /*
      * Check widget record:
      */
@@ -1061,6 +1053,17 @@ Tcl_Obj *Ttk_QueryStyle(
 	}
     }
 
+    /*
+     * Check dynamic settings:
+     */
+    stateMap = LookupStateMap(style, optionName);
+    if (stateMap) {
+	result = Ttk_StateMapLookup(NULL, stateMap, state);
+	if (result) {
+	    return result;
+	}
+    }
+
     /*
      * Use style default:
      */
@@ -1549,8 +1552,7 @@ static int StyleLayoutCmd(
     if (objc == 3) {
 	layoutTemplate = Ttk_FindLayoutTemplate(theme, layoutName);
 	if (!layoutTemplate) {
-	    Tcl_AppendResult(interp, "Layout ", layoutName, " not found",
-		    NULL);
+	    Tcl_AppendResult(interp, "Layout ", layoutName, " not found", NULL);
 	    return TCL_ERROR;
 	}
 	Tcl_SetObjResult(interp, Ttk_UnparseLayoutTemplate(layoutTemplate));
