@@ -1,5 +1,5 @@
 /*
- * treeview.c,v 1.26 2006/04/06 19:52:25 jenglish Exp
+ * treeview.c,v 1.31 2006/10/03 17:53:46 jenglish Exp
  * Copyright (c) 2004, Joe English
  *
  * Tile widget set: treeview widget.
@@ -51,6 +51,7 @@ struct TreeItemRec
     Tcl_Obj	*imageObj;
     Tcl_Obj	*valuesObj;
     Tcl_Obj	*openObj;
+    Tcl_Obj	*tagsObj;
 };
 
 static Tk_OptionSpec ItemOptionSpecs[] =
@@ -67,6 +68,9 @@ static Tk_OptionSpec ItemOptionSpecs[] =
     {TK_OPTION_BOOLEAN, "-open", "open", "Open",
 	"0", Tk_Offset(TreeItem,openObj), -1,
 	0,0,0 },
+    {TK_OPTION_STRING, "-tags", "tags", "Tags",
+	NULL, Tk_Offset(TreeItem,tagsObj), -1,
+	TK_OPTION_NULL_OK,0,0 },
 
     {TK_OPTION_END, 0,0,0, NULL, -1,-1, 0,0,0}
 };
@@ -86,6 +90,7 @@ static TreeItem *NewItem(void)
     item->imageObj = NULL;
     item->valuesObj = NULL;
     item->openObj = NULL;
+    item->tagsObj = NULL;
 
     return item;
 }
@@ -95,10 +100,11 @@ static TreeItem *NewItem(void)
  */
 static void FreeItem(TreeItem *item)
 {
-    if (item->textObj) Tcl_DecrRefCount(item->textObj);
-    if (item->imageObj) Tcl_DecrRefCount(item->imageObj);
-    if (item->valuesObj) Tcl_DecrRefCount(item->valuesObj);
-    if (item->openObj) Tcl_DecrRefCount(item->openObj);
+    if (item->textObj) { Tcl_DecrRefCount(item->textObj); }
+    if (item->imageObj) { Tcl_DecrRefCount(item->imageObj); }
+    if (item->valuesObj) { Tcl_DecrRefCount(item->valuesObj); }
+    if (item->openObj) { Tcl_DecrRefCount(item->openObj); }
+    if (item->tagsObj) { Tcl_DecrRefCount(item->tagsObj); }
     ckfree((ClientData)item);
 }
 
@@ -157,14 +163,51 @@ static TreeItem *NextPreorder(TreeItem *item)
     return item->next;
 }
 
+/*------------------------------------------------------------------------
+ * +++ Display items and tag options.
+ */
+
+typedef struct {
+    Tcl_Obj *textObj;		/* taken from item / data cell */
+    Tcl_Obj *imageObj;		/* taken from item */
+    Tcl_Obj *anchorObj;		/* from column */
+    Tcl_Obj *backgroundObj;	/* remainder from tag */
+    Tcl_Obj *foregroundObj;	
+    Tcl_Obj *fontObj;
+} DisplayItem;
+
+static Tk_OptionSpec TagOptionSpecs[] =
+{
+    {TK_OPTION_STRING, "-text", "text", "Text",
+	NULL, Tk_Offset(DisplayItem,textObj), -1,
+	TK_OPTION_NULL_OK,0,0 },
+    {TK_OPTION_STRING, "-image", "image", "Image",
+	NULL, Tk_Offset(DisplayItem,imageObj), -1,
+	TK_OPTION_NULL_OK,0,0 },
+    {TK_OPTION_ANCHOR, "-anchor", "anchor", "Anchor",
+	NULL, Tk_Offset(DisplayItem,anchorObj), -1, 
+	TK_OPTION_NULL_OK, 0, GEOMETRY_CHANGED},
+    {TK_OPTION_COLOR, "-background", "windowColor", "WindowColor", 
+	NULL, Tk_Offset(DisplayItem,backgroundObj), -1,
+	TK_OPTION_NULL_OK,0,0 },
+    {TK_OPTION_COLOR, "-foreground", "textColor", "TextColor", 
+	NULL, Tk_Offset(DisplayItem,foregroundObj), -1,
+	TK_OPTION_NULL_OK,0,0 },
+    {TK_OPTION_FONT, "-font", "font", "Font",
+	NULL, Tk_Offset(DisplayItem,fontObj), -1,
+	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
+
+    {TK_OPTION_END, 0,0,0, NULL, -1,-1, 0,0,0}
+};
+
+
+
 /*------------------------------------------------------------------------
  * +++ Columns.
  *
- * There are three separate option tables associated with the column record:
- * ColumnOptionSpecs for configuring the column,
- * HeadingOptionSpecs for drawing headings,
- * and DataCellOptionSpecs for drawing data cells.
- *
+ * There are separate option tables associated with the column record:
+ * ColumnOptionSpecs is for configuring the column,
+ * and HeadingOptionSpecs is for drawing headings.
  */
 typedef struct {
     int 	width;		/* Column width, in pixels */
@@ -204,14 +247,14 @@ static void InitColumn(TreeColumn *column)
 
 static void FreeColumn(TreeColumn *column)	/* @@@ rename */
 {
-    if (column->idObj) Tcl_DecrRefCount(column->idObj);
-    if (column->anchorObj) Tcl_DecrRefCount(column->anchorObj);
+    if (column->idObj) { Tcl_DecrRefCount(column->idObj); }
+    if (column->anchorObj) { Tcl_DecrRefCount(column->anchorObj); }
 
-    if (column->headingObj) Tcl_DecrRefCount(column->headingObj);
-    if (column->headingImageObj) Tcl_DecrRefCount(column->headingImageObj);
-    if (column->headingAnchorObj) Tcl_DecrRefCount(column->headingAnchorObj);
-    if (column->headingStateObj) Tcl_DecrRefCount(column->headingStateObj);
-    if (column->headingCommandObj) Tcl_DecrRefCount(column->headingCommandObj);
+    if (column->headingObj) { Tcl_DecrRefCount(column->headingObj); }
+    if (column->headingImageObj) { Tcl_DecrRefCount(column->headingImageObj); }
+    if (column->headingAnchorObj) { Tcl_DecrRefCount(column->headingAnchorObj); }
+    if (column->headingStateObj) { Tcl_DecrRefCount(column->headingStateObj); }
+    if (column->headingCommandObj) { Tcl_DecrRefCount(column->headingCommandObj); }
 
     /* Don't touch column->data, it's scratch storage */
 }
@@ -250,30 +293,6 @@ static Tk_OptionSpec HeadingOptionSpecs[] =
     {TK_OPTION_END, 0,0,0, NULL, -1,-1, 0,0,0}
 };
 
-static Tk_OptionSpec DataCellOptionSpecs[] =
-{
-    {TK_OPTION_STRING, "-text", "text", "Text",
-	"", Tk_Offset(TreeColumn,data), -1,
-	0,0,0 },
-    {TK_OPTION_ANCHOR, "-anchor", "anchor", "Anchor",
-	"w", Tk_Offset(TreeColumn,anchorObj), -1,
-	0,0,0 },
-    {TK_OPTION_END, 0,0,0, NULL, -1,-1, 0,0,0}
-};
-
-/*------------------------------------------------------------------------
- * +++ Row backgrounds.
- */
-
-typedef struct {
-    Tcl_Obj *unusedObj;
-} TreeRow;
-
-static Tk_OptionSpec RowOptionSpecs[] =
-{
-    {TK_OPTION_END, 0,0,0, NULL, -1,-1, 0,0,0}
-};
-
 /*------------------------------------------------------------------------
  * +++ -show option:
  * TODO: Implement SHOW_BRANCHES.
@@ -328,9 +347,10 @@ typedef struct
      */
     Tk_OptionTable itemOptionTable;
     Tk_OptionTable columnOptionTable;
-    Tk_OptionTable cellOptionTable;
     Tk_OptionTable headingOptionTable;
-    Tk_OptionTable rowOptionTable;
+    Tk_OptionTable tagOptionTable;
+    Tk_BindingTable bindingTable;
+    Ttk_TagTable tagTable;
 
     /* Acquired in GetLayout hook:
      */
@@ -359,6 +379,7 @@ typedef struct
     Tcl_Obj *paddingObj;	/* internal padding */
 
     Tcl_Obj *showObj;		/* -show list */
+    Tcl_Obj *selectModeObj;	/* -selectmode option */
 
     Scrollable yscroll;
     ScrollHandle yscrollHandle;
@@ -387,6 +408,8 @@ typedef struct {
 #define SCROLLCMD_CHANGED	(USER_MASK<<2)
 #define SHOW_CHANGED 		(USER_MASK<<3)
 
+static const char *SelectModeStrings[] = { "none", "browse", "extended", NULL };
+
 static Tk_OptionSpec TreeviewOptionSpecs[] =
 {
     WIDGET_TAKES_FOCUS,
@@ -401,6 +424,10 @@ static Tk_OptionSpec TreeviewOptionSpecs[] =
 	DEFAULT_SHOW, Tk_Offset(Treeview,tree.showObj), -1,
 	0,0,SHOW_CHANGED | GEOMETRY_CHANGED },
 
+    {TK_OPTION_STRING_TABLE, "-selectmode", "selectMode", "SelectMode",
+	"extended", Tk_Offset(Treeview,tree.selectModeObj), -1,
+	0,(ClientData)SelectModeStrings,0 },
+
     {TK_OPTION_PIXELS, "-height", "height", "Height",
 	DEF_TREE_ROWS, Tk_Offset(Treeview,tree.heightObj), -1,
 	0,0,GEOMETRY_CHANGED},
@@ -445,17 +472,6 @@ static Tcl_Obj *unshare(Tcl_Obj *objPtr)
     return objPtr;
 }
 
-/* @@@ Move this. */
-static Tcl_Obj *Ttk_NewBoxObj(Ttk_Box box)
-{
-    Tcl_Obj *result = Tcl_NewListObj(0,0);
-    Tcl_ListObjAppendElement(NULL, result, Tcl_NewIntObj(box.x));
-    Tcl_ListObjAppendElement(NULL, result, Tcl_NewIntObj(box.y));
-    Tcl_ListObjAppendElement(NULL, result, Tcl_NewIntObj(box.width));
-    Tcl_ListObjAppendElement(NULL, result, Tcl_NewIntObj(box.height));
-    return result;
-}
-
 /* DisplayLayout --
  * 	Rebind, place, and draw a layout + object combination.
  */
@@ -504,7 +520,8 @@ static int ColumnIndex(Tcl_Interp *interp, Treeview *tv, Tcl_Obj *columnIDObj)
     }
     Tcl_ResetResult(interp);
     Tcl_AppendResult(interp,
-	    "Invalid column index ", Tcl_GetString(columnIDObj), NULL);
+	"Invalid column index ", Tcl_GetString(columnIDObj),
+	NULL);
     return -1;
 }
 
@@ -716,6 +733,67 @@ static int TreeviewInitDisplayColumns(Tcl_Interp *interp, Treeview *tv)
     return TCL_OK;
 }
 
+/*------------------------------------------------------------------------
+ * +++ Event handlers.
+ */
+
+static TreeItem *IdentifyItem(Treeview *tv,int y,Ttk_Box *itemPos); /*forward*/
+
+const unsigned int TreeviewBindEventMask =
+      KeyPressMask|KeyReleaseMask
+    | ButtonPressMask|ButtonReleaseMask
+    | PointerMotionMask|ButtonMotionMask
+    | VirtualEventMask
+    ;
+
+static void TreeviewBindEventProc(void *clientData, XEvent *event)
+{
+    Treeview *tv = clientData;
+    TreeItem *item = NULL;
+    Ttk_Box unused;
+    void *taglist;
+    int nTags;
+
+    /*
+     * Figure out where to deliver the event.
+     */
+
+    switch (event->type) 
+    {
+	case KeyPress:
+	case KeyRelease:
+	case VirtualEvent:
+	    item = tv->tree.focus;
+	    break;
+	case ButtonPress:
+	case ButtonRelease:
+	    item = IdentifyItem(tv, event->xbutton.y, &unused);
+	    break;
+	case MotionNotify:
+	    item = IdentifyItem(tv, event->xmotion.y, &unused);
+	    break;
+	default:
+	    break;
+    }
+
+    if (!item) {
+	return;
+    }
+
+    /* ASSERT: Ttk_GetTagListFromObj returns TCL_OK. */
+    Ttk_GetTagListFromObj(NULL, tv->tree.tagTable, item->tagsObj,
+	    &nTags, &taglist);
+
+    /*
+     * Fire binding:
+     */
+    Tcl_Preserve(clientData);
+    Tk_BindEvent(tv->tree.bindingTable, event, tv->core.tkwin, nTags, taglist);
+    Tcl_Release(clientData);
+
+    Ttk_FreeTagList(taglist);
+}
+
 /*------------------------------------------------------------------------
  * +++ Initialization and cleanup.
  */
@@ -729,12 +807,16 @@ static int TreeviewInitialize(Tcl_Interp *interp, void *recordPtr)
 	Tk_CreateOptionTable(interp, ItemOptionSpecs);
     tv->tree.columnOptionTable =
 	Tk_CreateOptionTable(interp, ColumnOptionSpecs);
-    tv->tree.cellOptionTable =
-	Tk_CreateOptionTable(interp, DataCellOptionSpecs);
     tv->tree.headingOptionTable =
 	Tk_CreateOptionTable(interp, HeadingOptionSpecs);
-    tv->tree.rowOptionTable =
-	Tk_CreateOptionTable(interp, RowOptionSpecs);
+    tv->tree.tagOptionTable =
+	Tk_CreateOptionTable(interp, TagOptionSpecs);
+
+    tv->tree.tagTable = Ttk_CreateTagTable(
+	tv->tree.tagOptionTable, sizeof(DisplayItem));
+    tv->tree.bindingTable = Tk_CreateBindingTable(interp);
+    Tk_CreateEventHandler(tv->core.tkwin,
+	TreeviewBindEventMask, TreeviewBindEventProc, tv);
 
     tv->tree.itemLayout
 	= tv->tree.cellLayout
@@ -764,6 +846,8 @@ static int TreeviewInitialize(Tcl_Interp *interp, void *recordPtr)
     /* Create root item "":
      */
     tv->tree.root = NewItem();
+    Tk_InitOptions(interp, (ClientData)tv->tree.root,
+	tv->tree.itemOptionTable, tv->core.tkwin);
     tv->tree.root->entryPtr = Tcl_CreateHashEntry(&tv->tree.items, "", &unused);
     Tcl_SetHashValue(tv->tree.root->entryPtr, tv->tree.root);
 
@@ -778,6 +862,11 @@ static void TreeviewCleanup(void *recordPtr)
 {
     Treeview *tv = recordPtr;
 
+    Tk_DeleteEventHandler(tv->core.tkwin,
+	    TreeviewBindEventMask,  TreeviewBindEventProc, tv);
+    Tk_DeleteBindingTable(tv->tree.bindingTable);
+    Ttk_DeleteTagTable(tv->tree.tagTable);
+
     if (tv->tree.itemLayout) Ttk_FreeLayout(tv->tree.itemLayout);
     if (tv->tree.cellLayout) Ttk_FreeLayout(tv->tree.cellLayout);
     if (tv->tree.headingLayout) Ttk_FreeLayout(tv->tree.headingLayout);
@@ -882,6 +971,20 @@ static int ConfigureItem(
 	    item->state &= ~TTK_STATE_OPEN;
     }
 
+    /* Make sure -tags is a valid list
+     * (side effect: may create new tags)
+     */
+    if (item->tagsObj) {
+	void *taglist;
+	int nTags;
+	if (Ttk_GetTagListFromObj(interp, tv->tree.tagTable, item->tagsObj,
+		    &nTags, &taglist) != TCL_OK)
+	{
+	    goto error;
+	}
+	Ttk_FreeTagList(taglist);
+    }
+
     /* All OK.
      */
     Tk_FreeSavedOptions(&savedOptions);
@@ -1094,7 +1197,6 @@ static int IdentifyDisplayColumn(Treeview *tv, int x, int *x1)
     return -1;
 }
 
-
 /* + SubtreeHeight --
  * 	Returns the height of the visible subtree rooted at item.
  */
@@ -1127,7 +1229,7 @@ static int ItemYPosition(Treeview *tv, TreeItem *p)
 	    ypos += SubtreeHeight(p);
 	} else {
 	    p = p->parent;
-	    if (!(p && (p->state & TTK_STATE_OPEN))) {	
+	    if (!(p && (p->state & TTK_STATE_OPEN))) {
 		/* detached or closed ancestor */
 		return -1;
 	    }
@@ -1176,13 +1278,13 @@ static Ttk_Layout TreeviewGetLayout(
 
     if (!(
 	GetSublayout(interp, themePtr, treeLayout, ".Item",
-		tv->tree.itemOptionTable, &tv->tree.itemLayout)
+	    tv->tree.itemOptionTable, &tv->tree.itemLayout)
      && GetSublayout(interp, themePtr, treeLayout, ".Cell",
-		tv->tree.cellOptionTable, &tv->tree.cellLayout)
+	    tv->tree.tagOptionTable, &tv->tree.cellLayout)	/*@@@HERE*/
      && GetSublayout(interp, themePtr, treeLayout, ".Heading",
 	    tv->tree.headingOptionTable, &tv->tree.headingLayout)
      && GetSublayout(interp, themePtr, treeLayout, ".Row",
-	    tv->tree.rowOptionTable, &tv->tree.rowLayout)
+	    tv->tree.tagOptionTable, &tv->tree.rowLayout)	/*@@@HERE*/
     )) {
 	return 0;
     }
@@ -1282,11 +1384,43 @@ static void DrawHeadings(Treeview *tv, Drawable d, Ttk_Box b)
     }
 }
 
+/* + PrepareItem --
+ * 	Fill in a displayItem record from tag settings.
+ */
+static void PrepareItem(Treeview *tv, TreeItem *item, DisplayItem *displayItem)
+{
+    const int nOptions = sizeof(*displayItem)/sizeof(Tcl_Obj*);
+    Tcl_Obj **dest = (Tcl_Obj**)displayItem;
+    Tcl_Obj **objv = NULL;
+    int objc = 0;
+
+    memset(displayItem, 0, sizeof(*displayItem));
+
+    if (   item->tagsObj 
+    	&& Tcl_ListObjGetElements(NULL, item->tagsObj, &objc, &objv) == TCL_OK)
+    {
+	int i, j;
+	for (i=0; i<objc; ++i) {
+	    Ttk_Tag tag = Ttk_GetTagFromObj(tv->tree.tagTable, objv[i]);
+	    Tcl_Obj **tagRecord = Ttk_TagRecord(tag);
+
+	    if (tagRecord) {
+		for (j=0; j<nOptions; ++j) {
+		    if (tagRecord[j] != 0) {
+			dest[j] = tagRecord[j];
+		    }
+		}
+	    }
+	}
+    }
+}
+
 /* + DrawCells --
  *	Draw data cells for specified item.
  */
 static void DrawCells(
-    Treeview *tv, TreeItem *item, Drawable d, Ttk_Box b, int x, int y)
+    Treeview *tv, TreeItem *item, DisplayItem *displayItem,
+    Drawable d, Ttk_Box b, int x, int y)
 {
     Ttk_Layout layout = tv->tree.cellLayout;
     Ttk_State state = ItemState(tv, item);
@@ -1296,8 +1430,9 @@ static void DrawCells(
     Tcl_Obj **values = 0;
     int i;
 
-    if (!item->valuesObj)
+    if (!item->valuesObj) {
 	return;
+    }
 
     Tcl_ListObjGetElements(NULL, item->valuesObj, &nValues, &values);
     for (i = 0; i < tv->tree.nColumns; ++i) {
@@ -1308,7 +1443,11 @@ static void DrawCells(
 	TreeColumn *column = tv->tree.displayColumns[i];
 	Ttk_Box parcel = Ttk_PadBox(
 	    Ttk_MakeBox(b.x+x, b.y+y, column->width, height), cellPadding);
-	DisplayLayout(layout, column, state, parcel, d);
+
+	displayItem->textObj = column->data;
+	displayItem->anchorObj = column->anchorObj;
+
+	DisplayLayout(layout, displayItem, state, parcel, d);
 	x += column->width;
     }
 }
@@ -1322,19 +1461,20 @@ static void DrawItem(
 {
     Ttk_Layout layout = tv->tree.itemLayout;
     Ttk_State state = ItemState(tv, item);
+    DisplayItem displayItem;
     int height = ROWHEIGHT;
     int x = depth * INDENT;
     int y = (row - tv->tree.yscroll.first) * ROWHEIGHT;
 
     if (row % 2) state |= TTK_STATE_ALTERNATE;
 
+    PrepareItem(tv, item, &displayItem);
+
     /* Draw row background:
      */
     {
 	Ttk_Box rowBox = Ttk_MakeBox(b.x, b.y+y, TreeWidth(tv), height);
-	Ttk_Layout rowLayout =  tv->tree.rowLayout;
-	TreeRow row;	/* @@@ double-check this. */
-	DisplayLayout(rowLayout, &row, state, rowBox, d);
+	DisplayLayout(tv->tree.rowLayout, &displayItem, state, rowBox, d);
     }
 
     /* Draw tree label:
@@ -1350,7 +1490,7 @@ static void DrawItem(
 
     /* Draw data cells:
      */
-    DrawCells(tv, item, d, b, x, y);
+    DrawCells(tv, item, &displayItem, d, b, x, y);
 }
 
 /* + DrawSubtree --
@@ -1521,7 +1661,7 @@ static int RowNumber(Treeview *tv, TreeItem *item)
 
 /* + ItemDepth -- return the depth of a tree item.
  * 	The depth of an item is equal to the number of proper ancestors,
- * 	not counting the root node.  
+ * 	not counting the root node.
  */
 static int ItemDepth(TreeItem *item)
 {
@@ -1753,7 +1893,7 @@ static int TreeviewBBoxCommand(
     TreeColumn *column = 0;
     int ypos;
     Ttk_Box bbox;
- 
+
     if (objc < 3 || objc > 4) {
 	Tcl_WrongNumArgs(interp, 2, objv, "itemid ?column");
 	return TCL_ERROR;
@@ -1771,7 +1911,7 @@ static int TreeviewBBoxCommand(
      * ALTERNATE: (RowNumber(tv, item) - tv->tree.yscroll.first) * ROWHEIGHT;
      */
     ypos = ItemYPosition(tv, item) - (ROWHEIGHT * tv->tree.yscroll.first);
-    if (ypos < 0 || ypos > tv->tree.treeArea.height) { 
+    if (ypos < 0 || ypos > tv->tree.treeArea.height) {
 	/* not viewable, or off-screen */
 	return TCL_OK;
     }
@@ -1907,15 +2047,15 @@ static int TreeviewIdentifyCommand(
 	return TCL_ERROR;
     }
 
-    if (   Tcl_GetIndexFromObj(interp, objv[2], 
-		componentStrings, "componen", TCL_EXACT, &component) != TCL_OK
+    if (   Tcl_GetIndexFromObj(interp, objv[2],
+		componentStrings, "component", TCL_EXACT, &component) != TCL_OK
         || Tcl_GetIntFromObj(interp, objv[3], &x) != TCL_OK
 	|| Tcl_GetIntFromObj(interp, objv[4], &y) != TCL_OK
     ) {
 	return TCL_ERROR;
     }
 
-    switch (component) 
+    switch (component)
     {
 	case I_ROW :
 	{
@@ -2504,6 +2644,104 @@ static int TreeviewSelectionCommand(
     return TCL_OK;
 }
 
+/*------------------------------------------------------------------------
+ * +++ Widget commands -- tags and bindings.
+ */
+
+/* + $tv tag bind $tag ?$sequence ?$script??
+ */
+static int TreeviewTagBindCommand(
+    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
+{
+    Treeview *tv = recordPtr;
+    Ttk_Tag tag;
+
+    if (objc < 4 || objc > 6) {
+    	Tcl_WrongNumArgs(interp, 3, objv, "tagName ?sequence? ?script?");
+	return TCL_ERROR;
+    }
+
+    tag = Ttk_GetTagFromObj(tv->tree.tagTable, objv[3]);
+    if (!tag) { return TCL_ERROR; }
+
+    if (objc == 4) {		/* $tv tag bind $tag */
+	Tk_GetAllBindings(interp, tv->tree.bindingTable, tag);
+    } else if (objc == 5) { 	/* $tv tag bind $tag $sequence */
+	/* TODO: distinguish "no such binding" (OK) from "bad pattern" (ERROR)
+	 */
+	const char *script = Tk_GetBinding(interp,
+		tv->tree.bindingTable, tag, Tcl_GetString(objv[4]));
+	if (script != NULL) {
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(script,-1));
+	}
+    } else if (objc == 6) {	/* $tv tag bind $tag $sequence $script */
+	CONST char *sequence = Tcl_GetString(objv[4]);
+	CONST char *script = Tcl_GetString(objv[5]);
+	unsigned long mask = Tk_CreateBinding(interp,
+		tv->tree.bindingTable, tag, sequence, script, 0);
+
+	/* Test mask to make sure event is supported:
+	 */
+	if (mask & (~TreeviewBindEventMask)) {
+	    Tk_DeleteBinding(interp, tv->tree.bindingTable, tag, sequence);
+	    Tcl_ResetResult(interp);
+	    Tcl_AppendResult(interp, "unsupported event ", sequence,
+		"\nonly key, button, motion, and virtual events supported",
+		NULL);
+	    return TCL_ERROR;
+	}
+
+	return mask ? TCL_OK : TCL_ERROR;
+    }
+    return TCL_OK;
+}
+
+/* + $tv tag configure $tag ?-option ?value -option value...??
+ */
+static int TreeviewTagConfigureCommand(
+    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
+{
+    Treeview *tv = recordPtr;
+    void *tagRecord;
+    Ttk_Tag tag;
+
+    if (objc < 4) {
+    	Tcl_WrongNumArgs(interp, 3, objv, "tagName ?-option ?value ...??");
+	return TCL_ERROR;
+    }
+
+    tag = Ttk_GetTagFromObj(tv->tree.tagTable, objv[3]);
+    tagRecord = Ttk_TagRecord(tag);
+
+    if (objc == 4) {
+	return EnumerateOptions(interp, tagRecord, TagOptionSpecs,
+		tv->tree.tagOptionTable, tv->core.tkwin);
+    } else if (objc == 5) {
+	return GetOptionValue(interp, tagRecord, objv[4],
+		tv->tree.tagOptionTable, tv->core.tkwin);
+    }
+    /* else */
+    TtkRedisplayWidget(&tv->core);
+    return Tk_SetOptions(
+	interp, tagRecord, tv->tree.tagOptionTable,
+	objc - 4, objv + 4, tv->core.tkwin,
+	NULL/*savedOptions*/, NULL/*mask*/);
+}
+
+/* + $tv tag option args...
+ */
+static int TreeviewTagCommand(
+    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
+{
+    static WidgetCommandSpec TreeviewTagCommands[] = {
+	{ "bind",	TreeviewTagBindCommand },
+	{ "configure",	TreeviewTagConfigureCommand },
+	{0,0}
+    };
+    return WidgetEnsembleCommand(
+	    TreeviewTagCommands, 2, interp, objc, objv, recordPtr);
+}
+
 /*------------------------------------------------------------------------
  * +++ Widget commands record.
  */
@@ -2532,6 +2770,7 @@ static WidgetCommandSpec TreeviewCommands[] =
     { "selection" ,	TreeviewSelectionCommand },
     { "set",  		TreeviewSetCommand },
     { "state",  	WidgetStateCommand },
+    { "tag",    	TreeviewTagCommand },
     { "yview",  	TreeviewYViewCommand },
     { NULL, NULL }
 };
@@ -2636,7 +2875,7 @@ static void TreeitemIndicatorDraw(
     Drawable d, Ttk_Box b, Ttk_State state)
 {
     TreeitemIndicator *indicator = elementRecord;
-    ArrowDirection direction = 
+    ArrowDirection direction =
 	(state & TTK_STATE_OPEN) ? ARROW_DOWN : ARROW_RIGHT;
     Ttk_Padding margins;
     XColor *borderColor = Tk_GetColorFromObj(tkwin, indicator->colorObj);
@@ -2668,7 +2907,7 @@ static Ttk_ElementSpec TreeitemIndicatorElementSpec =
 };
 
 /*------------------------------------------------------------------------
- * +++ Row element:
+ * +++ Row element.
  */
 
 typedef struct
