@@ -1,4 +1,4 @@
-/* widget.c,v 1.43 2005/04/16 00:34:54 jenglish Exp
+/* widget.c,v 1.47 2005/10/08 15:53:42 jenglish Exp
  * Copyright (c) 2003, Joe English
  *
  * Tile widget implementation, core widget utilities.
@@ -31,10 +31,9 @@ static void UpdateLayout(Tcl_Interp *interp, WidgetCore *corePtr)
 
 static void UpdateGeometry(WidgetCore *corePtr)
 {
-    WidgetSpec *ws = corePtr->widgetSpec;
     int reqWidth = 1, reqHeight = 1;
 
-    if (ws->sizeProc(corePtr,&reqWidth,&reqHeight)) {
+    if (corePtr->widgetSpec->sizeProc(corePtr,&reqWidth,&reqHeight)) {
 	Tk_GeometryRequest(corePtr->tkwin, reqWidth, reqHeight);
     }
 }
@@ -43,8 +42,8 @@ static void UpdateGeometry(WidgetCore *corePtr)
  * RedisplayWidget --
  *	Redraw a widget.  Called as an idle handler.
  */
-static void
-RedisplayWidget(ClientData recordPtr)
+
+static void RedisplayWidget(ClientData recordPtr)
 {
     WidgetCore *corePtr = (WidgetCore *)recordPtr;
     Tk_Window tkwin = corePtr->tkwin;
@@ -52,7 +51,7 @@ RedisplayWidget(ClientData recordPtr)
     XGCValues gcValues;
     GC gc;
 
-    corePtr->flags &= ~(UPDATE_PENDING | REDISPLAY_REQUIRED);
+    corePtr->flags &= ~REDISPLAY_PENDING;
     if (!Tk_IsMapped(tkwin)) {
 	return;
     }
@@ -91,24 +90,34 @@ RedisplayWidget(ClientData recordPtr)
     Tk_FreeGC(Tk_Display(tkwin), gc);
 }
 
-/* WidgetChanged(corePtr, flags) --
- * 	Notification that the widget has changed.
- * 	if flags & RELAYOUT_REQUIRED, recompute geometry.
- * 	if flags & REDISPLAY_REQUIRED, schedule redisplay as an idle call.
+/* TtkRedisplayWidget -- 
+ * 	Schedule redisplay as an idle handler.
  */
-void WidgetChanged(WidgetCore *corePtr, unsigned flags)
+
+void TtkRedisplayWidget(WidgetCore *corePtr) 
 {
-    if (corePtr->flags & WIDGET_DESTROYED)
+    if (corePtr->flags & WIDGET_DESTROYED) {
 	return;
-
-    if (flags & RELAYOUT_REQUIRED)
-	UpdateGeometry(corePtr);
-
-    if (!(corePtr->flags & UPDATE_PENDING)) {
-	Tcl_DoWhenIdle(RedisplayWidget, (ClientData) corePtr);
-	corePtr->flags |= (UPDATE_PENDING | flags);
     }
 
+    if (!(corePtr->flags & REDISPLAY_PENDING)) {
+	Tcl_DoWhenIdle(RedisplayWidget, (ClientData) corePtr);
+	corePtr->flags |= REDISPLAY_PENDING;
+    }
+}
+
+/* TtkResizeWidget --
+ * 	Recompute widget size, schedule geometry propagation and redisplay. 
+ */
+
+void TtkResizeWidget(WidgetCore *corePtr)
+{
+    if (corePtr->flags & WIDGET_DESTROYED) {
+	return;
+    }
+
+    UpdateGeometry(corePtr);
+    TtkRedisplayWidget(corePtr);
 }
 
 /* WidgetEnsembleCommand --
@@ -230,27 +239,22 @@ static void CoreEventProc(ClientData clientData, XEvent *eventPtr)
 		UpdateGeometry(corePtr);
 		corePtr->flags |= WIDGET_REALIZED;
 	    }
-	    WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+	    TtkRedisplayWidget(corePtr);
 	    break;
 	case Expose :
 	    if (eventPtr->xexpose.count == 0) {
-		WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+		TtkRedisplayWidget(corePtr);
 	    }
 	    break;
 	case DestroyNotify :
-	    if (!corePtr->tkwin) {
-		/* All hell has broken loose */
-		Tcl_Panic("This is not supposed to happen");
-		return;
-	    }
-
 	    corePtr->flags |= WIDGET_DESTROYED;
 
 	    Tk_DeleteEventHandler(corePtr->tkwin,
 		    CoreEventMask,CoreEventProc,clientData);
 
-	    if (corePtr->flags & UPDATE_PENDING)
+	    if (corePtr->flags & REDISPLAY_PENDING) {
 		Tcl_CancelIdleCall(RedisplayWidget, clientData);
+	    }
 
 	    corePtr->widgetSpec->cleanupProc(corePtr);
 
@@ -282,22 +286,22 @@ static void CoreEventProc(ClientData clientData, XEvent *eventPtr)
 		    corePtr->state |= TTK_STATE_FOCUS;
 		else
 		    corePtr->state &= ~TTK_STATE_FOCUS;
-		WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+		TtkRedisplayWidget(corePtr);
 	    }
 	    break;
 	case ActivateNotify:
 	    corePtr->state &= ~TTK_STATE_BACKGROUND;
-	    WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+	    TtkRedisplayWidget(corePtr);
 	    break;
 	case DeactivateNotify:
 	    corePtr->state |= TTK_STATE_BACKGROUND;
-	    WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+	    TtkRedisplayWidget(corePtr);
 	    break;
 	case VirtualEvent:
 	    if (!strcmp("ThemeChanged", ((XVirtualEvent *)(eventPtr))->name)) {
 		UpdateLayout(corePtr->interp, corePtr);
 		UpdateGeometry(corePtr);
-		WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+		TtkRedisplayWidget(corePtr);
 	    }
 	default:
 	    /* can't happen... */
@@ -315,7 +319,7 @@ static void WidgetWorldChanged(ClientData clientData)
 {
     WidgetCore *corePtr = (WidgetCore*)clientData;
     UpdateGeometry(corePtr);
-    WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(corePtr);
 }
 
 static struct Tk_ClassProcs widgetClassProcs = {
@@ -391,11 +395,11 @@ int WidgetConstructorObjCmd(
     if (Tk_InitOptions(interp, recordPtr, optionTable, tkwin) != TCL_OK)
     	goto error;
 
-    if (Tk_SetOptions(interp, recordPtr, optionTable, objc - 2,
-	    objv + 2, tkwin, NULL/*savePtr*/, (int *)NULL/*maskPtr*/) != TCL_OK)
+    if (widgetSpec->initializeProc(interp, recordPtr) != TCL_OK)
 	goto error;
 
-    if (widgetSpec->initializeProc(interp, recordPtr) != TCL_OK)
+    if (Tk_SetOptions(interp, recordPtr, optionTable, objc - 2,
+	    objv + 2, tkwin, NULL/*savePtr*/, (int *)NULL/*maskPtr*/) != TCL_OK)
 	goto error;
 
     if (widgetSpec->configureProc(interp, recordPtr, ~0) != TCL_OK)
@@ -438,8 +442,9 @@ void WidgetChangeState(WidgetCore *corePtr,
 {
     Ttk_State oldState = corePtr->state;
     corePtr->state = (oldState & ~clearBits) | setBits;
-    if (corePtr->state ^ oldState)
-	WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+    if (corePtr->state ^ oldState) {
+	TtkRedisplayWidget(corePtr);
+    }
 }
 
 /*
@@ -663,7 +668,7 @@ Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 	    UpdateGeometry(corePtr);
 	}
 
-	WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+	TtkRedisplayWidget(corePtr);
 	result = Tcl_NewObj();
     }
 
@@ -709,7 +714,7 @@ int WidgetStateCommand(
     corePtr->state = Ttk_ModifyState(corePtr->state, &spec);
     changed = corePtr->state ^ oldState;
 
-    WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(corePtr);
 
     Tcl_SetObjResult(interp,
 	Ttk_NewStateSpecObj(oldState & changed, ~oldState & changed));
