@@ -1,5 +1,5 @@
 #
-# clamTheme.tcl,v 1.23 2005/09/29 15:34:54 jenglish Exp
+# clamTheme.tcl,v 1.25 2006/04/17 01:53:13 jenglish Exp
 #
 # Tile widget set: "Clam" theme
 #
@@ -69,9 +69,13 @@ namespace eval tile::theme::clam {
 	    ;
 
 	style configure TCheckbutton \
-	    -indicatorbackground "#ffffff" -indicatormargin {1 1 4 1}
+	    -indicatorbackground "#ffffff" \
+	    -indicatormargin {1 1 4 1} \
+	    -padding 2 ;
 	style configure TRadiobutton \
-	    -indicatorbackground "#ffffff" -indicatormargin {1 1 4 1}
+	    -indicatorbackground "#ffffff" \
+	    -indicatormargin {1 1 4 1} \
+	    -padding 2 ;
 	style map TCheckbutton -indicatorbackground \
 	    [list  disabled $colors(-frame)  pressed $colors(-frame)]
 	style map TRadiobutton -indicatorbackground \
@@ -104,7 +108,7 @@ namespace eval tile::theme::clam {
 	    ;
 
     	style configure TLabelframe \
-	    -labeloutside true -labelinset 0 -labelspace 2 \
+	    -labeloutside true -labelmargins {0 0 0 4} \
 	    -borderwidth 2 -relief raised
 
 	style configure TProgressbar -background $colors(-frame)
