@@ -1,5 +1,5 @@
 #
-# defaults.tcl,v 1.30 2006/02/21 15:45:47 jenglish Exp
+# defaults.tcl,v 1.31 2006/08/21 19:01:36 jenglish Exp
 #
 # Tile widget set: Default theme
 #
@@ -73,8 +73,7 @@ namespace eval tile {
 
 	style configure TNotebook.Tab \
 	    -padding {4 2} -background $colors(-darker)
-	style map TNotebook.Tab -background \
-	    [list selected $colors(-frame) active $colors(-activebg)]
+	style map TNotebook.Tab -background [list selected $colors(-frame)]
 
 	#
 	# Toolbar buttons:
