@@ -1,5 +1,5 @@
 #
-# dialog.tcl,v 1.2 2005/11/02 22:14:44 jenglish Exp
+# dialog.tcl,v 1.4 2006/09/13 14:05:07 jenglish Exp
 #
 # Copyright (c) 2005, Joe English.  Freely redistributable.
 #
@@ -89,7 +89,7 @@ proc ttk::dialog::Constructor {dlg args} {
     	-message	""
 	-detail		""
 	-command	ttk::dialog::nop
-	-icon 		info
+	-icon 		""
 	-buttons 	{}
 	-labels 	{}
 	-default 	{}
@@ -144,14 +144,17 @@ proc ttk::dialog::Constructor {dlg args} {
     # Build dialog:
     #
     if {$options(-parent) ne ""} {
-    	wm transient $dlg $parent
+    	wm transient $dlg $options(-parent)
     }
     wm title $dlg $options(-title)
     wm protocol $dlg WM_DELETE_WINDOW { }
 
     set f [ttk::frame $dlg.f] 
 
-    ttk::label $f.icon -image [tile::stockIcon dialog/$options(-icon)]
+    ttk::label $f.icon 
+    if {$options(-icon) ne ""} {
+	$f.icon configure -image [tile::stockIcon dialog/$options(-icon)]
+    }
     ttk::label $f.message -textvariable ${dlg}(-message) \
     	-font TkCaptionFont -wraplength $Config(textwidth)\
 	-anchor w -justify left
@@ -182,6 +185,8 @@ proc ttk::dialog::Constructor {dlg args} {
     if {$options(-cancel) ne ""} {
 	bind $dlg <KeyPress-Escape> \
 	    [list event generate $cmd.$options(-cancel) <<Invoke>>]
+	wm protocol $dlg WM_DELETE_WINDOW \
+	    [list event generate $cmd.$options(-cancel) <<Invoke>>]
     }
 
     #
@@ -200,8 +205,10 @@ proc ttk::dialog::Constructor {dlg args} {
 	    -padx $Config(margin)
     }
 
-    pack $f.icon -side left -anchor n -expand false \
-    	-pady $Config(margin) -padx $Config(margin)
+    if {$options(-icon) ne ""} {
+	pack $f.icon -side left -anchor n -expand false \
+	    -pady $Config(margin) -padx $Config(margin)
+    }
 
     pack $f.message -side top -expand false -fill x \
     	-padx $Config(margin) -pady $Config(margin)
@@ -243,7 +250,7 @@ proc ttk::dialog::Done {dlg button} {
     } elseif {$rc == 3 || $rc == 4} {
 	# break or continue -- don't dismiss dialog
 	return
-    } 
+    }
     dismiss $dlg
 }
 
