@@ -1,5 +1,5 @@
 #
-# icons.tcl,v 1.2 2006/03/16 01:11:29 jenglish Exp
+# icons.tcl,v 1.3 2006/08/20 17:27:36 jenglish Exp
 #
 # Tile package -- stock icons.
 # 
@@ -29,9 +29,14 @@ proc tile::stockIcon {name} {
     return $Icons($name)
 }
 
-proc tile::DefineIcon {name data} {
+# defineImage --
+#	Define a new stock icon.
+# 
+proc tile::defineImage {name args} {
     variable Icons
-    set Icons($name) [image create photo ::tile::icons::$name -data $data]
+    set iconName ::tile::icons::$name
+    eval [linsert $args 0 image create photo $iconName]
+    set Icons($name) $iconName
 }
 
 #
@@ -39,7 +44,7 @@ proc tile::DefineIcon {name data} {
 #
 # SOURCE: dialog icons taken from BWidget toolkit.
 #
-tile::DefineIcon dialog/error {
+tile::defineImage dialog/error -data {
     R0lGODlhIAAgALMAAIQAAISEhPf/Mf8AAP//////////////////////////
     /////////////////////yH5BAEAAAIALAAAAAAgACAAAASwUMhJBbj41s0n
     HmAIYl0JiCgKlNWVvqHGnnA9mnY+rBytw4DAxhci2IwqoSdFaMKaSBFPQhxA
@@ -48,7 +53,7 @@ tile::DefineIcon dialog/error {
     phWKOra3G3YuqReJwiwUiRkZwsPEuMnNycslzrIdEQAAOw==
 }
 
-tile::DefineIcon dialog/info {
+tile::defineImage dialog/info -data {
     R0lGODlhIAAgALMAAAAAAAAA/4SEhMbGxvf/Mf//////////////////////
     /////////////////////yH5BAEAAAQALAAAAAAgACAAAAStkMhJibj41s0n
     HkUoDljXXaCoqqRgUkK6zqP7CvQQ7IGsAiYcjcejFYAb4ZAYMB4rMaeO51sN
@@ -57,7 +62,7 @@ tile::DefineIcon dialog/info {
     GJMhs7WoIoC7v7i+v7uTwsO1o5HHu7TLtcodEQAAOw==
 }
 
-tile::DefineIcon dialog/question {
+tile::defineImage dialog/question -data {
     R0lGODlhIAAgALMAAAAAAAAA/4SEhMbGxvf/Mf//////////////////////
     /////////////////////yH5BAEAAAQALAAAAAAgACAAAAS2kMhJibj41s0n
     HkUoDljXXaCoqqRgUkK6zqP7CnS+AiY+D4GgUKbibXwrYEoYIIqMHmcoqGLS
@@ -66,7 +71,7 @@ tile::DefineIcon dialog/question {
     GaI3F7MyoaYvHhicoLe/sk8axcnCisnKBczNxa3I0cW+1bm/EQAAOw==
 }
 
-tile::DefineIcon dialog/warning {
+tile::defineImage dialog/warning -data {
     R0lGODlhIAAgALMAAAAAAISEAISEhMbGxv//AP//////////////////////
     /////////////////////yH5BAEAAAUALAAAAAAgACAAAASrsMhJZ7g16y0D
     IQPAjZr3gYBAroV5piq7uWcoxHJFv3eun0BUz9cJAmHElhFow8lcIQBgwHOu
@@ -75,7 +80,7 @@ tile::DefineIcon dialog/warning {
     s4sqtqswp2W+v743whTCxcbHyG0FyczJEhEAADs=
 }
 
-tile::DefineIcon dialog/auth {
+tile::defineImage dialog/auth -data {
     R0lGODlhIAAgAIQAAAAA/wAAAICAgICAAP///7CwsMDAwMjIAPjIAOjo6Pj4
     AODg4HBwcMj4ANjY2JiYANDQ0MjIyPj4yKCgoMiYAMjImDAwAMjIMJiYmJCQ
     kAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAAAALAAAAAAgACAAAAX+ICCOYmCa
@@ -89,7 +94,7 @@ tile::DefineIcon dialog/auth {
     oJUrxi04TCzA0PQsWh9kMVx1u6UFA3116zLJGwIAOw==
 }
 
-tile::DefineIcon dialog/busy {
+tile::defineImage dialog/busy -data {
     R0lGODlhIAAgALMAAAAAAIAAAACAAICAAAAAgIAAgACAgMDAwICAgP8AAAD/
     AP//AAAA//8A/wD//////yH5BAEAAAsALAAAAAAgACAAAASAcMlJq7046827
     /2AYBmRpkoC4BMlzvEkspypg3zitIsfjvgcEQifi+X7BoUpi9AGFxFATCV0u
