@@ -1,5 +1,5 @@
 #
-# notebook.tcl,v 1.18 2006/04/08 00:48:43 jenglish Exp
+# notebook.tcl,v 1.19 2006/07/04 17:21:55 jenglish Exp
 # 
 # Bindings for TNotebook widget
 #
@@ -32,6 +32,7 @@ proc ttk::notebook::ActivateTab {w tab} {
 	focus $w
     } else {
     	$w select $tab
+	update ;# needed so focus logic sees correct mapped/unmapped states
 	if {[set f [ttk::focusFirst [$w select]]] ne ""} {
 	    keynav::traverseTo $f
 	}
@@ -40,10 +41,12 @@ proc ttk::notebook::ActivateTab {w tab} {
 
 # ttk::focusFirst $w --
 #	Return the first descendant of $w, in preorder traversal order,
-#	that wants to take keyboard focus, "" if none do.
+#	that can take keyboard focus, "" if none do.
+#
+# See also: tk_focusNext
 #
 proc ttk::focusFirst {w} {
-    if {[tile::wantsFocus $w]} { 
+    if {[tile::takesFocus $w]} { 
 	return $w 
     }
     foreach child [winfo children $w] {
