@@ -1,5 +1,5 @@
 #
-# tile.tcl,v 1.87 2006/01/20 20:00:25 jenglish Exp
+# tile.tcl,v 1.90 2006/10/03 18:23:03 jenglish Exp
 #
 # Tile widget set initialization script.
 #
@@ -22,6 +22,14 @@ source [file join $tile::library cursors.tcl]
 source [file join $tile::library icons.tcl]
 source [file join $tile::library utils.tcl]
 
+### Forward-compatibility.
+#
+# In 0.8.0, ttk::paned will be renamed ttk::panedwindow.
+#
+# Alias new name to current name to ease the transition:
+#
+interp alias {} ::ttk::panedwindow {} ::ttk::paned
+
 ### Deprecated aliases.
 #
 
@@ -131,8 +139,9 @@ source [file join $tile::library progress.tcl]
 source [file join $tile::library notebook.tcl]
 source [file join $tile::library paned.tcl]
 source [file join $tile::library entry.tcl]
-source [file join $tile::library combobox.tcl]
+source [file join $tile::library combobox.tcl]	;# dependency: entry.tcl
 source [file join $tile::library treeview.tcl]
+source [file join $tile::library sizegrip.tcl]
 source [file join $tile::library dialog.tcl]
 
 ## Label and Labelframe bindings:
