@@ -1,5 +1,5 @@
 #
-# xpTheme.tcl,v 1.32 2006/04/17 03:29:45 jenglish Exp
+# xpTheme.tcl,v 1.33 2006/09/30 17:51:24 jenglish Exp
 #
 # Tile widget set: XP Native theme
 #
@@ -39,6 +39,12 @@ namespace eval tile {
 	    -selectforeground [list !focus SystemWindowText] \
 	    ;
 	style configure TCombobox -padding 2
+	style map TCombobox \
+	    -selectbackground [list !focus SystemWindow] \
+	    -selectforeground [list !focus SystemWindowText] \
+	    -foreground	[list {readonly focus} SystemHighlightText] \
+	    -focusfill	[list {readonly focus} SystemHighlight] \
+	    ;
 
 	style configure Toolbutton -padding {4 4}
     }
