@@ -1,5 +1,5 @@
 /*
- * aquaTheme.c,v 1.33 2005/11/21 06:55:57 jenglish Exp
+ * aquaTheme.c,v 1.34 2006/07/29 21:27:54 jenglish Exp
  *
  * Tk theme engine for Mac OSX, using the Appearance Manager API.
  *
@@ -679,6 +679,53 @@ static Ttk_ElementSpec SeparatorElementSpec =
     SeparatorElementDraw
 };
 
+/*----------------------------------------------------------------------
+ * +++ Size grip element.
+ */
+static const ThemeGrowDirection sizegripGrowDirection 
+	= kThemeGrowRight|kThemeGrowDown;
+
+static void SizegripElementSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+{
+    Point origin;
+    Rect bounds;
+
+    origin.h = origin.v = 0;
+    GetThemeStandaloneGrowBoxBounds(
+    	origin, sizegripGrowDirection, false, &bounds);
+    *widthPtr = bounds.right - bounds.left;
+    *heightPtr = bounds.bottom - bounds.top;
+}
+
+static void SizegripElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
+{
+    Point origin;
+    origin.h = b.x; origin.v = b.y;
+
+    /* Grow box only supports kThemeStateActive, kThemeStateInactive */
+    state &= TTK_STATE_BACKGROUND;
+
+    BEGIN_DRAWING(d)
+    DrawThemeStandaloneGrowBox(
+    	origin, sizegripGrowDirection, false,
+	Ttk_StateTableLookup(ThemeStateTable, state));
+    END_DRAWING
+}
+
+static Ttk_ElementSpec SizegripElementSpec =
+{
+    TK_STYLE_VERSION_2,
+    sizeof(NullElement),
+    NullElementOptions,
+    SizegripElementSize,
+    SizegripElementDraw
+};
+
+
 /*----------------------------------------------------------------------
  * +++ Background element -- an experiment.
  *
@@ -918,6 +965,8 @@ int AquaTheme_Init(Tcl_Interp *interp)
     Ttk_RegisterElementSpec(themePtr, "hseparator",&SeparatorElementSpec,0);
     Ttk_RegisterElementSpec(themePtr, "vseparator",&SeparatorElementSpec,0);
 
+    Ttk_RegisterElementSpec(themePtr, "sizegrip",&SizegripElementSpec,0);
+
     /* <<NOTE-TRACKS>>
      * The Progressbar widget adjusts the size of the pbar element.
      * In the Aqua theme, the appearance manager computes the bar geometry;
