@@ -1,4 +1,4 @@
-/* monitor.c,v 1.14 2005/09/11 00:09:02 jenglish Exp
+/* monitor.c,v 1.15 2005/12/02 21:51:22 hobbs Exp
  */
 
 #ifdef _MSC_VER
@@ -98,7 +98,7 @@ CreateThemeMonitorWindow(HINSTANCE hinst, Tcl_Interp *interp)
 	hwnd = CreateWindow( name, title, WS_OVERLAPPEDWINDOW,
 	    CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
 	    NULL, NULL, hinst, NULL );
-	SetWindowLong(hwnd, GWL_USERDATA, (LONG)interp);
+	SetWindowLongPtr(hwnd, GWLP_USERDATA, (LONG)interp);
 	ShowWindow(hwnd, SW_HIDE);
 	UpdateWindow(hwnd);
     }
@@ -115,7 +115,7 @@ DestroyThemeMonitorWindow(void *clientData)
 static LRESULT WINAPI
 WndProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
 {
-    Tcl_Interp *interp = (Tcl_Interp *)GetWindowLong(hwnd, GWL_USERDATA);
+    Tcl_Interp *interp = (Tcl_Interp *)GetWindowLongPtr(hwnd, GWLP_USERDATA);
     Ttk_Theme theme;
 
     switch (msg) {
