@@ -1,6 +1,6 @@
 /* winTheme.c - Copyright (C) 2004 Pat Thoyts <patthoyts@users.sf.net>
  *
- * winTheme.c,v 1.33 2006/04/06 20:02:53 jenglish Exp
+ * winTheme.c,v 1.36 2006/10/02 22:48:46 jenglish Exp
  */
 
 #ifdef _MSC_VER
@@ -123,6 +123,9 @@ static FrameControlElementData FrameControlElements[] =
     { "rightarrow",
 	DFC_SCROLL, DFCS_SCROLLRIGHT, SM_CXHSCROLL, SM_CYHSCROLL,
 	arrow_statemap, {0,0,0,0} },
+    { "sizegrip",
+    	DFC_SCROLL, DFCS_SCROLLSIZEGRIP, SM_CXVSCROLL, SM_CYHSCROLL,
+	arrow_statemap, {0,0,0,0} },
 
     { 0,0,0,0,0,0, {0,0,0,0} }
 };
@@ -400,24 +403,58 @@ static Ttk_ElementSpec FocusElementSpec =
     FocusElementDraw
 };
 
+/* FillFocusElement --
+ * 	Draws a focus ring filled with the selection color
+ */
+
+typedef struct {
+    Tcl_Obj *fillColorObj;
+} FillFocusElement;
+
+static Ttk_ElementOptionSpec FillFocusElementOptions[] = {
+    { "-focusfill", TK_OPTION_COLOR,
+	Tk_Offset(FillFocusElement,fillColorObj), "white" },
+    { NULL }
+};
+
+	/* @@@ FIX THIS */
+static void FillFocusElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
+{
+    FillFocusElement *focus = elementRecord;
+    if (state & TTK_STATE_FOCUS) {
+	RECT rc = BoxToRect(b);
+	TkWinDCState dcState;
+	XColor *fillColor = Tk_GetColorFromObj(tkwin, focus->fillColorObj);
+	GC gc = Tk_GCForColor(fillColor, d);
+	HDC hdc;
+
+	XFillRectangle(Tk_Display(tkwin),d,gc, b.x,b.y,b.width,b.height);
+	hdc = TkWinGetDrawableDC(Tk_Display(tkwin), d, &dcState);
+    	DrawFocusRect(hdc, &rc);
+	TkWinReleaseDrawableDC(d, hdc, &dcState);
+    }
+}
+
 /*
  * ComboboxFocusElement --
- * 	Read-only comboboxes have a focus ring, editable ones do not.
+ * 	Read-only comboboxes have a filled focus ring, editable ones do not.
  */
 static void ComboboxFocusElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     Drawable d, Ttk_Box b, unsigned int state)
 {
     if (state & TTK_STATE_READONLY) {
-    	FocusElementDraw(clientData, elementRecord, tkwin, d, b, state);
+    	FillFocusElementDraw(clientData, elementRecord, tkwin, d, b, state);
     }
 }
 
 static Ttk_ElementSpec ComboboxFocusElementSpec =
 {
     TK_STYLE_VERSION_2,
-    sizeof(NullElement),
-    NullElementOptions,
+    sizeof(FillFocusElement),
+    FillFocusElementOptions,
     FocusElementGeometry,
     ComboboxFocusElementDraw
 };
